# -*- coding: utf-8 -*-
#
# Python version Copyright (c) 2015 John Stowers
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

import cv2
import numpy as np

from Box2D import (b2Color, b2DistanceJoint, b2MouseJoint, b2PulleyJoint)
from Box2D.b2 import (staticBody, dynamicBody, kinematicBody, polygonShape,
                      circleShape, loopShape, edgeShape)


def cvcolor(color):
    return int(255.0 * color[2]), int(255.0 * color[1]), int(255.0 * color[0])


def cvcoord(pos):
    return tuple(map(int, pos))


class OpencvDrawFuncs(object):

    def __init__(self, w, h, ppm, fill_polygon=True, flip_y=True):
        self._w = w
        self._h = h
        self._ppm = ppm
        self._colors = {
            staticBody: (255, 255, 255),
            dynamicBody: (127, 127, 127),
            kinematicBody: (127, 127, 230),
        }
        self._fill_polygon = fill_polygon
        self._flip_y = flip_y
        self.screen = np.zeros((self._h, self._w, 3), np.uint8)

    def install(self):
        polygonShape.draw = self._draw_polygon
        circleShape.draw = self._draw_circle
        loopShape.draw = self._draw_loop
        edgeShape.draw = self._draw_edge

    def draw_world(self, world):
        for body in world.bodies:
            for fixture in body.fixtures:
                fixture.shape.draw(body, fixture)
        for joint in world.joints:
            self._draw_joint(joint)

    def clear_screen(self, screen=None):
        if screen is None:
            self.screen.fill(0)
        else:
            self.screen = screen

    def _fix_vertices(self, vertices):
        if self._flip_y:
            return [(v[0], self._h - v[1]) for v in vertices]
        else:
            return [(v[0], v[1]) for v in vertices]

    def _draw_joint(self, joint):
        bodyA, bodyB = joint.bodyA, joint.bodyB
        xf1, xf2 = bodyA.transform, bodyB.transform
        x1, x2 = xf1.position, xf2.position
        p1, p2 = joint.anchorA, joint.anchorB
        color = b2Color(0.5, 0.8, 0.8)

        x1, x2, p1, p2 = self._fix_vertices((x1 * self._ppm, x2 * self._ppm,
                                             p1 * self._ppm, p2 * self._ppm))

        if isinstance(joint, b2DistanceJoint):
            cv2.line(self.screen, cvcoord(p1), cvcoord(p2), cvcolor(color), 1)
        elif isinstance(joint, b2PulleyJoint):
            s1, s2 = joint.groundAnchorA, joint.groundAnchorB
            s1, s2 = self._fix_vertices((s1 * self._ppm, s2 * self._ppm))
            cv2.line(self.screen, cvcoord(s1), cvcoord(p1), cvcolor(color), 1)
            cv2.line(self.screen, cvcoord(s2), cvcoord(p2), cvcolor(color), 1)
            cv2.line(self.screen, cvcoord(s1), cvcoord(s2), cvcolor(color), 1)
        elif isinstance(joint, b2MouseJoint):
            pass  # don't draw it here
        else:
            cv2.line(self.screen, cvcoord(x1), cvcoord(p1), cvcolor(color), 1)
            cv2.line(self.screen, cvcoord(p1), cvcoord(p2), cvcolor(color), 1)
            cv2.line(self.screen, cvcoord(x2), cvcoord(p2), cvcolor(color), 1)

    def _draw_polygon(self, body, fixture):
        polygon = fixture.shape

        transform = body.transform
        vertices = self._fix_vertices([transform * v * self._ppm
                                       for v in polygon.vertices])

        pts = np.array(vertices, np.int32)
        pts = pts.reshape((-1, 1, 2))
        cv2.polylines(self.screen, [pts], True, self._colors[body.type])

        if self._fill_polygon:
            lightc = np.array(self._colors[body.type], dtype=int) * 0.5
            cv2.fillPoly(self.screen, [pts], lightc)

    def _draw_circle(self, body, fixture):
        circle = fixture.shape
        position = self._fix_vertices(
            [body.transform * circle.pos * self._ppm])[0]
        cv2.circle(self.screen, cvcoord(position), int(
            circle.radius * self._ppm), self._colors[body.type], 1)

    def _draw_edge(self, body, fixture):
        edge = fixture.shape
        v = [body.transform * edge.vertex1 * self._ppm,
             body.transform * edge.vertex2 * self._ppm]
        vertices = self._fix_vertices(v)
        cv2.line(self.screen, cvcoord(vertices[0]),
                 cvcoord(vertices[1]), self._colors[body.type], 1)

    def _draw_loop(self, body, fixture):
        loop = fixture.shape
        transform = body.transform
        vertices = self._fix_vertices([transform * v * self._ppm
                                       for v in loop.vertices])
        v1 = vertices[-1]
        for v2 in vertices:
            cv2.line(self.screen, cvcoord(v1), cvcoord(v2),
                     self._colors[body.type], 1)
            v1 = v2



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import Box2D
import sys

class testWorld (unittest.TestCase):
    def setUp(self):
        pass

    def test_world(self):
        try:
            world = Box2D.b2World(Box2D.b2Vec2(0.0, -10.0), True)
            world = Box2D.b2World((0.0, -10.0), True)
            world = Box2D.b2World([0.0, -10.0], False)
            world = Box2D.b2World([0.0, -10.0])
            world = Box2D.b2World()
            world = Box2D.b2World(gravity=[0.0, -10.0])
        except Exception:
            self.fail("Failed to create world (%s)" % sys.exc_info()[1])

    def test_helloworld(self):
        gravity = Box2D.b2Vec2(0, -10)
         
        doSleep = True
         
        world = Box2D.b2World(gravity, doSleep)

        groundBodyDef = Box2D.b2BodyDef()
        groundBodyDef.position = [0, -10]
         
        groundBody = world.CreateBody(groundBodyDef)
         
        groundBox = Box2D.b2PolygonShape()
         
        groundBox.SetAsBox(50, 10)
         
        groundBody.CreateFixturesFromShapes(groundBox)
         
        bodyDef = Box2D.b2BodyDef()
        bodyDef.type = Box2D.b2_dynamicBody
        bodyDef.position = (0, 4)
        body = world.CreateBody(bodyDef)
         
        dynamicBox = Box2D.b2PolygonShape()
        dynamicBox.SetAsBox(1, 1)

        fixtureDef = Box2D.b2FixtureDef()
        fixtureDef.shape = dynamicBox

        fixtureDef.density = 1
         
        fixtureDef.friction = 0.3
         
        body.CreateFixture(fixtureDef)
         
        timeStep = 1.0 / 60
        vel_iters, pos_iters = 6, 2

        for i in range(60):
            world.Step(timeStep, vel_iters, pos_iters)
            world.ClearForces()

if __name__ == '__main__':
    unittest.main()




"""Defines the top-level application widget"""

import pygame
from pygame.locals import *

from . import pguglobals
from . import container
from .const import *

class App(container.Container):
    """The top-level widget for an application.
    
    Example:
        import pygame
        from pgu import gui

        widget = gui.Button("Testing")

        app = gui.App()
        app.init(widget=widget)
        app.run()

    """

    # The top-level widget in the application
    widget = None
    # The pygame display for rendering the GUI. Note this may be a subsurface
    # of the full surface.
    screen = None
    # The region of the (full) pygame display that contains the GUI. If set,
    # this is used when transforming the mouse position from screen 
    # coordinates into the subsurface coordinates.
    appArea = None

    def __init__(self, theme=None, **params):
        """Create a new application given the (optional) theme instance."""
        self.set_global_app()

        if theme == None: 
            from .theme import Theme
            theme = Theme()
        self.theme = theme
        
        params['decorate'] = 'app'
        container.Container.__init__(self,**params)
        self._quit = False
        self.widget = None
        self._chsize = False
        self._repaint = False
        
        self.screen = None
        self.container = None

    def set_global_app(self):
        """Registers this app as _the_ global PGU application. You 
        generally shouldn't need to call this function."""
        # Keep a global reference to this application instance so that PGU
        # components can easily find it.
        pguglobals.app = self
        # For backwards compatibility we keep a reference in the class 
        # itself too.
        App.app = self
        
    def resize(self):
        if self.screen:
            # The user has explicitly specified a screen surface
            size = self.screen.get_size()

        elif pygame.display.get_surface():
            # Use the existing pygame display
            self.screen = pygame.display.get_surface()
            size = self.screen.get_size()

        else:
            # Otherwise we must allocate a new pygame display
            if self.style.width != 0 and self.style.height != 0:
                # Create a new screen based on the desired app size
                size = (self.style.width, self.style.height)
        
            else:
                # Use the size of the top-most widget
                size = self.widget.rect.size = self.widget.resize()
            # Create the display
            self.screen = pygame.display.set_mode(size, SWSURFACE)

        #use screen to set up size of this widget
        self.style.width,self.style.height = size
        self.rect.size = size
        self.rect.topleft = (0, 0)
        
        self.widget.rect.topleft = (0, 0)
        self.widget.rect.size = self.widget.resize(*size)

        for w in self.windows:
            w.rect.size = w.resize()

        self._chsize = False

    def init(self, widget=None, screen=None, area=None):
        """Initialize the application.

        Keyword arguments:
            widget -- the top-level widget in the application
            screen -- the pygame surface to render to
            area -- the rectangle (within 'screen') to use for rendering
        """

        self.set_global_app()
        
        if (widget): 
            # Set the top-level widget
            self.widget = widget
        if (screen): 
            if (area):
                # Take a subsurface of the given screen
                self.appArea = area
                self.screen = screen.subsurface(area)
            else:
                # Use the entire screen for the app
                self.screen = screen
        
        self.resize()   
        
        w = self.widget     
        
        self.widgets = []
        self.widgets.append(w)
        w.container = self
        self.focus(w)
        
        pygame.key.set_repeat(500,30)
        
        self._repaint = True
        self._quit = False
        
        self.send(INIT)
    
    def event(self,ev):
        """Pass an event to the main widget. If you are managing your own
        mainloop, this function should be called periodically when you are
        processing pygame events.
        """
        self.set_global_app()

        if (self.appArea and hasattr(ev, "pos")):
            # Translate into subsurface coordinates
            pos = (ev.pos[0]-self.appArea.x,
                   ev.pos[1]-self.appArea.y)
            args = {"pos" : pos}
            # Copy over other misc mouse parameters
            for name in ("buttons", "rel", "button"):
                if (hasattr(ev, name)):
                    args[name] = getattr(ev, name)
            
            ev = pygame.event.Event(ev.type, args)

        #NOTE: might want to deal with ACTIVEEVENT in the future.
        self.send(ev.type, ev)
        container.Container.event(self, ev)
        if ev.type == MOUSEBUTTONUP:
            if ev.button not in (4,5): # Ignores the mouse wheel
                # Also issue a "CLICK" event
                sub = pygame.event.Event(CLICK,{
                    'button' : ev.button,
                    'pos' : ev.pos})
                self.send(sub.type,sub)
                container.Container.event(self,sub)
    
    def loop(self):
        """Performs one iteration of the PGU application loop, which
        processes events and update the pygame display."""
        self.set_global_app()

        for e in pygame.event.get():
            if not (e.type == QUIT and self.mywindow):
                self.event(e)
        rects = self.update(self.screen)
        pygame.display.update(rects)
        
        
    def paint(self,screen=None):
        """Renders the application onto the given pygame surface"""
        if (screen):
            self.screen = screen

        if self._chsize:
            self._chsize = False
            self.resize()

        if self.background:
            self.background.paint(self.screen)

        container.Container.paint(self, self.screen)

    def update(self,screen=None):
        """Update the screen in a semi-efficient manner, and returns
        a list of pygame rects to be updated."""
        if (screen):
            self.screen = screen

        if self._chsize:
            self.resize()
            self._chsize = False
            return None

        if self._repaint:
            self.paint(self.screen)
            self._repaint = False
            rects = [pygame.Rect(0, 0,
                                 self.screen.get_width(),
                                 self.screen.get_height())]
        else:
            rects = container.Container.update(self,self.screen)

        if (self.appArea):
            # Translate the rects from subsurface coordinates into
            # full display coordinates.
            for r in rects:
                r.move_ip(self.appArea.topleft)

        return rects
    
    def run(self, widget=None, screen=None, delay=10): 
        """Run an application.
        
        Automatically calls App.init and then forever loops while
        calling App.event and App.update

        Keyword arguments:
            widget -- the top-level widget to use
            screen -- the pygame surface to render to
            delay -- the delay between updates (in milliseconds)
        """
        self.init(widget,screen)
        while not self._quit:
            self.loop()
            pygame.time.wait(delay)

    def reupdate(self,w=None): 
        pass

    def repaint(self,w=None): 
        self._repaint = True

    def repaintall(self): 
        self._repaint = True

    def chsize(self):
        if (not self._chsize):
            self._chsize = True
            self._repaint = True
    
    def quit(self,value=None): 
        self._quit = True

    def open(self, w, pos=None):
        """Opens the given PGU window and positions it on the screen"""
        w.container = self
        
        if (w.rect.w == 0 or w.rect.h == 0):
            w.rect.size = w.resize()
        
        if (not pos): 
            # Auto-center the window
            w.rect.center = self.rect.center
        else: 
            # Show the window in a particular location
            w.rect.topleft = pos
        
        self.windows.append(w)
        self.mywindow = w
        self.focus(w)
        self.repaint(w)
        w.send(OPEN)

    def close(self, w):
        """Closes the previously opened PGU window"""
        if self.myfocus is w: self.blur(w)

        if w not in self.windows: return #no need to remove it twice! happens.
        
        self.windows.remove(w)
        
        self.mywindow = None
        if self.windows:
            self.mywindow = self.windows[-1]
            self.focus(self.mywindow)
        
        if not self.mywindow:
            self.myfocus = self.widget #HACK: should be done fancier, i think..
            if not self.myhover:
                self.enter(self.widget)
         
        self.repaintall()
        w.send(CLOSE)


class Desktop(App):
    """Create an App using the desktop theme class."""
    def __init__(self,**params):
        params.setdefault('cls','desktop')
        App.__init__(self,**params)




"""
"""
from . import widget

class Form(widget.Widget):
    """A form that automatically will contain all named widgets.
    
    After a form is created, all named widget that are subsequently created are 
    added to that form.  You may use dict style access to access named widgets.
    
    Example:

        f = gui.Form()
        
        w = gui.Input("Phil",name="firstname")
        w = gui.Input("Hassey",name="lastname")
        
        print f.results()
        print ''
        print f.items()
        print ''
        print f['firstname'].value
        print f['lastname'].value

    """

    # The current form instance
    form = None
    # The list of PGU widgets that are tracked by this form
    _elist = None
    # A mapping of PGU widgets tracked by this form (name -> instance)
    _emap = None
    # The dirty flag is set when a new widget is added to the form
    _dirty = 0
    
    def __init__(self):
        widget.Widget.__init__(self,decorate=False)
        self._elist = []
        self._emap = {}
        self._dirty = 0
        # Register this form as the one used by new widgets
        Form.form = self
    
    def add(self,e,name=None,value=None):
        """Adds a PGU widget to this form"""
        if name != None: e.name = name
        if value != None: e.value = value
        self._elist.append(e)
        self._dirty = 1
    
    def _clean(self):
        # Remove elements from our list if they no longer have an assigned name
        for e in self._elist[:]:
            if not hasattr(e,'name') or e.name == None:
                self._elist.remove(e)
        # Update the name-to-widget mapping
        self._emap = {}
        for e in self._elist:
            self._emap[e.name] = e
        self._dirty = 0
    
    def __getitem__(self,k):
        """Returns the widget instance given the name of the widget"""
        if self._dirty: self._clean()
        return self._emap[k]
    
    def __contains__(self,k):
        """Returns true if this form contains the named widget"""
        if self._dirty: self._clean()
        if k in self._emap: return True
        return False
    
    def results(self):
        """Return a dict of name, widget-value pairs."""
        if self._dirty: self._clean()
        r = {}
        for e in self._elist:
            # Make sure the widget has a 'value' (eg tables do not)
            if (hasattr(e, "value")):
                r[e.name] = e.value
            else:
                r[e.name] = None
        return r
    
    def items(self):
        """Return a list of name, widget pairs."""
        return self.results().items()
    




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from math import sin

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2Color, b2EdgeShape, b2FixtureDef, b2PolygonShape)


class MotorJoint (Framework):
    name = "MotorJoint"
    description = 'g to stop/go'
    count = 800

    def __init__(self):
        Framework.__init__(self)

        ground = self.world.CreateStaticBody(
            shapes=[b2EdgeShape(vertices=[(-20, 0), (20, 0)])],
        )

        # Define motorized body
        body = self.world.CreateDynamicBody(
            position=(0, 8),
            allowSleep=False,
            fixtures=b2FixtureDef(density=2.0, friction=0.6,
                                  shape=b2PolygonShape(box=(2.0, 0.5)),),
        )

        self.joint = self.world.CreateMotorJoint(bodyA=ground, bodyB=body,
                                                 maxForce=1000, maxTorque=1000)

        self.go = False
        self.time = 0.0

    def Keyboard(self, key):
        if key == Keys.K_g:
            self.go = not self.go

    def Step(self, settings):
        Framework.Step(self, settings)

        if self.go and settings.hz > 0.0:
            self.time += 1.0 / settings.hz

        linear_offset = (6 * sin(2.0 * self.time), 8.0 +
                         4.0 * sin(1.0 * self.time))
        angular_offset = 4.0 * self.time

        self.joint.linearOffset = linear_offset
        self.joint.angularOffset = angular_offset

        renderer = self.renderer
        renderer.DrawPoint(renderer.to_screen(
            linear_offset), 4, b2Color(0.9, 0.9, 0.9))

if __name__ == "__main__":
    main(MotorJoint)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.


from Box2D.examples.framework import (Framework, main)
from Box2D import (b2FixtureDef, b2PolygonShape, b2_pi)


class Tumbler (Framework):
    name = "Tumbler"
    description = ''
    count = 800

    def __init__(self):
        Framework.__init__(self)

        ground = self.world.CreateBody()

        body = self.world.CreateDynamicBody(
            position=(0, 10),
            allowSleep=False,
            shapeFixture=b2FixtureDef(density=5.0),
            shapes=[
                b2PolygonShape(box=(0.5, 10, (10, 0), 0)),
                b2PolygonShape(box=(0.5, 10, (-10, 0), 0)),
                b2PolygonShape(box=(10, 0.5, (0, 10), 0)),
                b2PolygonShape(box=(10, 0.5, (0, -10), 0)),
            ]
        )

        self.joint = self.world.CreateRevoluteJoint(bodyA=ground, bodyB=body,
                                                    localAnchorA=(0, 10), localAnchorB=(0, 0),
                                                    referenceAngle=0, motorSpeed=0.05 * b2_pi,
                                                    enableMotor=True, maxMotorTorque=1.0e8)

    def Step(self, settings):
        Framework.Step(self, settings)

        self.count -= 1
        if self.count == 0:
            return

        self.world.CreateDynamicBody(
            position=(0, 10),
            allowSleep=False,
            fixtures=b2FixtureDef(
                density=1.0, shape=b2PolygonShape(box=(0.125, 0.125))),
        )

if __name__ == "__main__":
    main(Tumbler)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2CircleShape, b2EdgeShape, b2FixtureDef, b2PolygonShape)


def create_bridge(world, ground, size, offset, plank_count, friction=0.6, density=1.0):
    """
    Create a bridge with plank_count planks,
    utilizing rectangular planks of size (width, height).
    The bridge should start at x_offset, and continue to
    roughly x_offset+width*plank_count.
    The y will not change.
    """
    width, height = size
    x_offset, y_offset = offset
    half_height = height / 2
    plank = b2FixtureDef(
        shape=b2PolygonShape(box=(width / 2, height / 2)),
        friction=friction,
        density=density,
    )

    bodies = []
    prevBody = ground
    for i in range(plank_count):
        body = world.CreateDynamicBody(
            position=(x_offset + width * i, y_offset),
            fixtures=plank,
        )
        bodies.append(body)

        world.CreateRevoluteJoint(
            bodyA=prevBody,
            bodyB=body,
            anchor=(x_offset + width * (i - 0.5), y_offset)
        )

        prevBody = body

    world.CreateRevoluteJoint(
        bodyA=prevBody,
        bodyB=ground,
        anchor=(x_offset + width * (plank_count - 0.5), y_offset),
    )
    return bodies


class Bridge (Framework):
    name = "Bridge"
    numPlanks = 30  # Number of planks in the bridge

    def __init__(self):
        super(Bridge, self).__init__()

        # The ground
        ground = self.world.CreateBody(
            shapes=b2EdgeShape(vertices=[(-40, 0), (40, 0)])
        )

        create_bridge(self.world, ground, (1.0, 0.25),
                      (-14.5, 5), self.numPlanks, 0.2, 20)

        fixture = b2FixtureDef(
            shape=b2PolygonShape(vertices=[(-0.5, 0.0),
                                           (0.5, 0.0),
                                           (0.0, 1.5),
                                           ]),
            density=1.0
        )
        for i in range(2):
            self.world.CreateDynamicBody(
                position=(-8 + 8 * i, 12),
                fixtures=fixture,
            )

        fixture = b2FixtureDef(shape=b2CircleShape(radius=0.5), density=1)
        for i in range(3):
            self.world.CreateDynamicBody(
                position=(-6 + 6 * i, 10),
                fixtures=fixture,
            )

if __name__ == "__main__":
    main(Bridge)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D.b2 import (edgeShape, circleShape, fixtureDef, polygonShape)


class Pulley (Framework):
    name = "Pulley"

    def __init__(self):
        super(Pulley, self).__init__()
        y, L, a, b = 16.0, 12.0, 1.0, 2.0
        # The ground
        ground = self.world.CreateStaticBody(
            shapes=[edgeShape(vertices=[(-40, 0), (40, 0)]),
                    circleShape(radius=2, pos=(-10.0, y + b + L)),
                    circleShape(radius=2, pos=(10.0, y + b + L))]
        )

        bodyA = self.world.CreateDynamicBody(
            position=(-10, y),
            fixtures=fixtureDef(shape=polygonShape(box=(a, b)), density=5.0),
        )
        bodyB = self.world.CreateDynamicBody(
            position=(10, y),
            fixtures=fixtureDef(shape=polygonShape(box=(a, b)), density=5.0),
        )

        self.pulley = self.world.CreatePulleyJoint(
            bodyA=bodyA,
            bodyB=bodyB,
            anchorA=(-10.0, y + b),
            anchorB=(10.0, y + b),
            groundAnchorA=(-10.0, y + b + L),
            groundAnchorB=(10.0, y + b + L),
            ratio=1.5,
        )

    def Step(self, settings):
        super(Pulley, self).Step(settings)

        ratio = self.pulley.ratio
        L = self.pulley.length1 + self.pulley.length2 * ratio
        self.Print('L1 + %4.2f * L2 = %4.2f' % (ratio, L))

if __name__ == "__main__":
    main(Pulley)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pyqt4_gui.ui'
#
# Created: Sun Mar 13 19:19:05 2011
#      by: PyQt4 UI code generator 4.7.6
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(800, 600)
        MainWindow.setTabShape(QtGui.QTabWidget.Rounded)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.graphicsView = QtGui.QGraphicsView(self.centralwidget)
        self.graphicsView.setObjectName(_fromUtf8("graphicsView"))
        self.horizontalLayout.addWidget(self.graphicsView)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 21))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.mnuFile = QtGui.QMenu(self.menubar)
        self.mnuFile.setObjectName(_fromUtf8("mnuFile"))
        self.menu_Options = QtGui.QMenu(self.menubar)
        self.menu_Options.setObjectName(_fromUtf8("menu_Options"))
        self.menu_Font_size = QtGui.QMenu(self.menu_Options)
        self.menu_Font_size.setObjectName(_fromUtf8("menu_Font_size"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.dwProperties = QtGui.QDockWidget(MainWindow)
        self.dwProperties.setObjectName(_fromUtf8("dwProperties"))
        self.dwcProperties = QtGui.QWidget()
        self.dwcProperties.setObjectName(_fromUtf8("dwcProperties"))
        self.verticalLayout = QtGui.QVBoxLayout(self.dwcProperties)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.gbOptions = QtGui.QGroupBox(self.dwcProperties)
        self.gbOptions.setObjectName(_fromUtf8("gbOptions"))
        self.verticalLayout.addWidget(self.gbOptions)
        self.twProperties = QtGui.QTableWidget(self.dwcProperties)
        self.twProperties.setLineWidth(1)
        self.twProperties.setAlternatingRowColors(True)
        self.twProperties.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.twProperties.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.twProperties.setObjectName(_fromUtf8("twProperties"))
        self.twProperties.setColumnCount(0)
        self.twProperties.setRowCount(0)
        self.verticalLayout.addWidget(self.twProperties)
        self.dwProperties.setWidget(self.dwcProperties)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dwProperties)
        self.mnuExit = QtGui.QAction(MainWindow)
        self.mnuExit.setObjectName(_fromUtf8("mnuExit"))
        self.mnuIncreaseFontSize = QtGui.QAction(MainWindow)
        self.mnuIncreaseFontSize.setObjectName(_fromUtf8("mnuIncreaseFontSize"))
        self.mnuDecreaseFontSize = QtGui.QAction(MainWindow)
        self.mnuDecreaseFontSize.setObjectName(_fromUtf8("mnuDecreaseFontSize"))
        self.mnuFile.addAction(self.mnuExit)
        self.menu_Font_size.addAction(self.mnuIncreaseFontSize)
        self.menu_Font_size.addAction(self.mnuDecreaseFontSize)
        self.menu_Options.addAction(self.menu_Font_size.menuAction())
        self.menubar.addAction(self.mnuFile.menuAction())
        self.menubar.addAction(self.menu_Options.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "pybox2d testbed", None, QtGui.QApplication.UnicodeUTF8))
        self.mnuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Options.setTitle(QtGui.QApplication.translate("MainWindow", "&Options", None, QtGui.QApplication.UnicodeUTF8))
        self.menu_Font_size.setTitle(QtGui.QApplication.translate("MainWindow", "&Font size", None, QtGui.QApplication.UnicodeUTF8))
        self.dwProperties.setWindowTitle(QtGui.QApplication.translate("MainWindow", "Properties", None, QtGui.QApplication.UnicodeUTF8))
        self.gbOptions.setTitle(QtGui.QApplication.translate("MainWindow", "Options", None, QtGui.QApplication.UnicodeUTF8))
        self.twProperties.setSortingEnabled(True)
        self.mnuExit.setText(QtGui.QApplication.translate("MainWindow", "E&xit", None, QtGui.QApplication.UnicodeUTF8))
        self.mnuIncreaseFontSize.setText(QtGui.QApplication.translate("MainWindow", "&Increase", None, QtGui.QApplication.UnicodeUTF8))
        self.mnuDecreaseFontSize.setText(QtGui.QApplication.translate("MainWindow", "&Decrease", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from math import sqrt

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2CircleShape, b2FixtureDef, b2PolygonShape, b2Random,
                   b2Vec2, b2_epsilon)

# ***** NOTE *****
# ***** NOTE *****
# This example does not appear to be working currently...
# It was ported from the JBox2D (Java) version
# ***** NOTE *****
# ***** NOTE *****


class Liquid (Framework):
    name = "Liquid Test"
    description = ''
    bullet = None

    num_particles = 1000
    total_mass = 10.0

    fluid_minx = -11.0
    fluid_maxx = 5.0
    fluid_miny = -10.0
    fluid_maxy = 10.0

    hash_width = 40
    hash_height = 40

    rad = 0.6
    visc = 0.004

    def __init__(self):
        super(Liquid, self).__init__()

        self.per_particle_mass = self.total_mass / self.num_particles

        ground = self.world.CreateStaticBody(
            shapes=[
                b2PolygonShape(box=[5.0, 0.5]),
                b2PolygonShape(box=[1.0, 0.2, (0, 4), -0.2]),
                b2PolygonShape(box=[1.5, 0.2, (-1.2, 5.2), -1.5]),
                b2PolygonShape(box=[0.5, 50.0, (5, 0), 0.0]),
                b2PolygonShape(box=[0.5, 3.0, (-8, 0), 0.0]),
                b2PolygonShape(box=[2.0, 0.1, (-6, -2.8), 0.1]),
                b2CircleShape(radius=0.5, pos=(-.5, -4)),
            ]
        )

        cx = 0
        cy = 25
        box_width = 2.0
        box_height = 20.0
        self.liquid = []
        for i in range(self.num_particles):
            self.createDroplet((b2Random(cx - box_width * 0.5,
                                         cx + box_width * 0.5),
                                b2Random(cy - box_height * 0.5,
                                         cy + box_height * 0.5)))

        self.createBoxSurfer()

        if hasattr(self, 'settings'):
            self.settings.enableSubStepping = False

    def createBoxSurfer(self):
        self.surfer = self.world.CreateDynamicBody(position=(0, 25))
        self.surfer.CreatePolygonFixture(
            density=1,
            box=(b2Random(0.3, 0.7), b2Random(0.3, 0.7)),
        )

    def createDroplet(self, position):
        body = self.world.CreateDynamicBody(
            position=position,
            fixedRotation=True,
            allowSleep=False,
        )
        body.CreateCircleFixture(
            groupIndex=-10,
            radius=0.05,
            restitution=0.4,
            friction=0,
        )
        body.mass = self.per_particle_mass
        self.liquid.append(body)

    def applyLiquidConstraint(self, dt):
        # (original comments left untouched)
        # Unfortunately, this simulation method is not actually scale
        # invariant, and it breaks down for rad < ~3 or so.  So we need
        # to scale everything to an ideal rad and then scale it back after.

        idealRad = 50
        idealRad2 = idealRad ** 2
        multiplier = idealRad / self.rad
        info = dict([(drop, (drop.position, multiplier * drop.position,
                             multiplier * drop.linearVelocity))
                     for drop in self.liquid])
        change = dict([(drop, b2Vec2(0, 0)) for drop in self.liquid])
        dx = self.fluid_maxx - self.fluid_minx
        dy = self.fluid_maxy - self.fluid_miny
        range_ = (-1, 0, 1)
        hash_width = self.hash_width
        hash_height = self.hash_height
        max_len = 9.9e9
        visc = self.visc
        hash = self.hash
        neighbors = set()
        # Populate the neighbor list from the 9 nearest cells
        for drop, ((worldx, worldy), (mx, my), (mvx, mvy)) in list(info.items()):
            hx = int((worldx / dx) * hash_width)
            hy = int((worldy / dy) * hash_height)
            neighbors.clear()
            for nx in range_:
                xc = hx + nx
                if not (0 <= xc < hash_width):
                    continue

                for ny in range_:
                    yc = hy + ny
                    if yc in hash[xc]:
                        for neighbor in hash[xc][yc]:
                            neighbors.add(neighbor)

            if drop in neighbors:
                neighbors.remove(drop)

            # Particle pressure calculated by particle proximity
            # Pressures = 0 iff all particles within range are idealRad
            # distance away
            lengths = []
            p = 0
            pnear = 0
            for neighbor in neighbors:
                nx, ny = info[neighbor][1]
                vx, vy = nx - mx, ny - my
                if -idealRad < vx < idealRad and -idealRad < vy < idealRad:
                    len_sqr = vx ** 2 + vy ** 2
                    if len_sqr < idealRad2:
                        len_ = sqrt(len_sqr)
                        if len_ < b2_epsilon:
                            len_ = idealRad - 0.01
                        lengths.append(len_)
                        oneminusq = 1.0 - (len_ / idealRad)
                        sq = oneminusq ** 2
                        p += sq
                        pnear += sq * oneminusq
                else:
                    lengths.append(max_len)

            # Now actually apply the forces
            pressure = (p - 5) / 2.0  # normal pressure term
            presnear = pnear / 2.0  # near particles term
            changex, changey = 0, 0
            for len_, neighbor in zip(lengths, neighbors):
                (nx, ny), (nvx, nvy) = info[neighbor][1:3]
                vx, vy = nx - mx, ny - my
                if -idealRad < vx < idealRad and -idealRad < vy < idealRad:
                    if len_ < idealRad:
                        oneminusq = 1.0 - (len_ / idealRad)
                        factor = oneminusq * \
                            (pressure + presnear * oneminusq) / (2.0 * len_)
                        dx_, dy_ = vx * factor, vy * factor
                        relvx, relvy = nvx - mvx, nvy - mvy

                        factor = visc * oneminusq * dt
                        dx_ -= relvx * factor
                        dy_ -= relvy * factor
                        change[neighbor] += (dx_, dy_)
                        changex -= dx_
                        changey -= dy_

            change[drop] += (changex, changey)

        for drop, (dx_, dy_) in list(change.items()):
            if dx_ != 0 or dy_ != 0:
                drop.position += (dx_ / multiplier, dy_ / multiplier)
                drop.linearVelocity += (dx_ / (multiplier * dt),
                                        dy_ / (multiplier * dt))

    def hashLocations(self):
        hash_width = self.hash_width
        hash_height = self.hash_height

        self.hash = hash = dict([(i, {}) for i in range(hash_width)])
        info = [(drop, drop.position) for drop in self.liquid]

        dx = self.fluid_maxx - self.fluid_minx
        dy = self.fluid_maxy - self.fluid_miny
        xs, ys = set(), set()
        for drop, (worldx, worldy) in info:
            hx = int((worldx / dx) * hash_width)
            hy = int((worldy / dy) * hash_height)
            xs.add(hx)
            ys.add(hy)
            if 0 <= hx < hash_width and 0 <= hy < hash_height:
                x = hash[hx]
                if hy not in x:
                    x[hy] = [drop]
                else:
                    x[hy].append(drop)

    def dampenLiquid(self):
        for drop in self.liquid:
            drop.linearVelocity *= 0.995

    def checkBounds(self):
        self.hash = None

        to_remove = [
            drop for drop in self.liquid if drop.position.y < self.fluid_miny]
        for drop in to_remove:
            self.liquid.remove(drop)
            self.world.DestroyBody(drop)

            self.createDroplet(
                (0.0 + b2Random(-0.6, 0.6), 15.0 + b2Random(-2.3, 2.0)))

        if self.surfer.position.y < -15:
            self.world.DestroyBody(self.surfer)
            self.createBoxSurfer()

    def Step(self, settings):
        super(Liquid, self).Step(settings)

        dt = 1.0 / settings.hz
        self.hashLocations()
        self.applyLiquidConstraint(dt)
        self.dampenLiquid()
        self.checkBounds()

    def Keyboard(self, key):
        if key == Keys.K_b:
            if self.bullet:
                self.world.DestroyBody(self.bullet)
                self.bullet = None
            circle = b2FixtureDef(
                shape=b2CircleShape(radius=0.25),
                density=20,
                restitution=0.05)
            self.bullet = self.world.CreateDynamicBody(
                position=(-31, 5),
                bullet=True,
                fixtures=circle,
                linearVelocity=(400, 0),
            )

if __name__ == "__main__":
    main(Liquid)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A small self contained example showing how to use OpencvDrawFuncs
to integrate pybox2d into an opencv mainloop

In short:
One static body:
    + One fixture: big polygon to represent the ground
Two dynamic bodies:
    + One fixture: a polygon
    + One fixture: a circle
And some drawing code that extends the shape classes.

John Stowers
"""
import cv2

import Box2D
from Box2D.b2 import (polygonShape, world)
from opencv_draw import OpencvDrawFuncs

# --- constants ---
# Box2D deals with meters, but we want to display pixels,
# so define a conversion factor:
TARGET_FPS = 60
TIME_STEP = 1.0 / TARGET_FPS

# --- pybox2d world setup ---
# Create the world
world = world(gravity=(0, -10), doSleep=True)

# And a static body to hold the ground shape
ground_body = world.CreateStaticBody(
    position=(0, 0),
    shapes=polygonShape(box=(50, 1)),
)

# Create a couple dynamic bodies
bodyc = world.CreateDynamicBody(position=(20, 45))
circle = bodyc.CreateCircleFixture(radius=0.5, density=1, friction=0.3)

bodyb = world.CreateDynamicBody(position=(30, 45), angle=15)
box = bodyb.CreatePolygonFixture(box=(2, 1), density=1, friction=0.3)

world.CreateWeldJoint(bodyA=bodyc, bodyB=bodyb, anchor=bodyb.worldCenter)

drawer = OpencvDrawFuncs(w=640, h=480, ppm=20)
drawer.install()

while True:
    key = 0xFF & cv2.waitKey(int(TIME_STEP * 1000))  # milliseconds
    if key == 27:
        break
    drawer.clear_screen()

    drawer.draw_world(world)

    # Make Box2D simulate the physics of our world for one step.
    world.Step(TIME_STEP, 10, 10)

    # Flip the screen and try to keep at the target FPS
    cv2.imshow("world", drawer.screen)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2EdgeShape, b2FixtureDef, b2PolygonShape)


class Mobile (Framework):
    name = "Mobile"
    max_depth = 4

    def __init__(self):
        Framework.__init__(self)

        ground = self.world.CreateStaticBody(
            position=(0, 20),
            shapes=[b2EdgeShape(vertices=[(-20, 0), (20, 0)])],
        )

        a = 0.5
        depth = 0
        self.root = self.add_node(ground, (0, 0), depth, 3.0, a)

        self.world.CreateRevoluteJoint(bodyA=ground, bodyB=self.root,
                                       localAnchorA=(0, 0), localAnchorB=(0, a))

    def add_node(self, parent, local_anchor, depth, offset, a):
        density = 20.0
        h = (0, a)

        p = parent.position + local_anchor - h

        fixture = b2FixtureDef(shape=b2PolygonShape(box=(0.25 * a, a)),
                               density=density)
        body = self.world.CreateDynamicBody(position=p, fixtures=fixture)

        if depth == self.max_depth:
            return body

        a1 = (offset, -a)
        a2 = (-offset, -a)
        body1 = self.add_node(body, a1, depth + 1, 0.5 * offset, a)
        body2 = self.add_node(body, a2, depth + 1, 0.5 * offset, a)

        self.world.CreateRevoluteJoint(bodyA=body, bodyB=body1,
                                       localAnchorA=a1, localAnchorB=h)
        self.world.CreateRevoluteJoint(bodyA=body, bodyB=body2,
                                       localAnchorA=a2, localAnchorB=h)

        return body

if __name__ == "__main__":
    main(Mobile)



"""
"""
import pygame
from pygame.locals import *

from .const import *
from . import widget

class TextArea(widget.Widget):
    """A multi-line text input.
    
    Example:
        w = TextArea(value="Cuzco the Goat",size=20)
        w = TextArea("Marbles")
        w = TextArea("Groucho\nHarpo\nChico\nGummo\nZeppo\n\nMarx", 200, 400, 12)
    
    """
    def __init__(self,value="",width = 120, height = 30, size=20,**params):
        params.setdefault('cls','input')
        params.setdefault('width', width)
        params.setdefault('height', height)
        
        widget.Widget.__init__(self,**params)
        self.value = value                # The value of the TextArea
        self.pos = len(str(value))        # The position of the cursor
        self.vscroll = 0                # The number of lines that the TextArea is currently scrolled
        self.font = self.style.font        # The font used for rendering the text
        self.cursor_w = 2                 # Cursor width (NOTE: should be in a style)
        w,h = self.font.size("e"*size)    
        if not self.style.height: self.style.height = h
        if not self.style.width: self.style.width = w
    
## BUG: This causes textarea to grow every time table._Table_td calculates its
## size.
##    def resize(self,width=None,height=None):
##        if (width != None) and (height != None):
##            print 'TextArea RESIZE'
##            self.rect = pygame.Rect(self.rect.x, self.rect.y, width, height)
##        return self.rect.w, self.rect.h
        
    def paint(self,s):
        
        # TODO: What's up with this 20 magic number? It's the margin of the left and right sides, but I'm not sure how this should be gotten other than by trial and error.
        max_line_w = self.rect.w - 20
                
        # Update the line allocation for the box's value
        self.doLines(max_line_w)
        
        # Make sure that the vpos and hpos of the cursor is set properly
        self.updateCursorPos()

        # Make sure that we're scrolled vertically such that the cursor is visible
        if (self.vscroll < 0):
            self.vscroll = 0
        if (self.vpos < self.vscroll):
            self.vscroll = self.vpos
        elif ((self.vpos - self.vscroll + 1) * self.line_h > self.rect.h):
            self.vscroll = - (self.rect.h / self.line_h - self.vpos - 1)

        # Blit each of the lines in turn
        cnt = 0
        for line in self.lines:
            line_pos = (0, (cnt - self.vscroll) * self.line_h)
            if (line_pos[1] >= 0) and (line_pos[1] < self.rect.h):
                s.blit( self.font.render(line, 1, self.style.color), line_pos )
            cnt += 1
        
        # If the textarea is focused, then also show the cursor
        if self.container.myfocus is self:
            r = self.getCursorRect()
            s.fill(self.style.color,r)
    
    # This function updates self.vpos and self.hpos based on self.pos
    def updateCursorPos(self):
        self.vpos = 0 # Reset the current line that the cursor is on
        self.hpos = 0
        
        line_cnt = 0
        char_cnt = 0

        for line in self.lines:
            line_char_start = char_cnt # The number of characters at the start of the line
            
            # Keep track of the character count for words
            char_cnt += len(line)
            
            # If our cursor count is still less than the cursor position, then we can update our cursor line to assume that it's at least on this line
            if (char_cnt > self.pos):
                self.vpos = line_cnt
                self.hpos = self.pos - line_char_start

                break # Now that we know where our cursor is, we exit the loop

            line_cnt += 1
        
        if (char_cnt <= self.pos) and (len(self.lines) > 0):
            self.vpos = len(self.lines) - 1
            self.hpos = len(self.lines[ self.vpos ] )

    # Returns a rectangle that is of the size and position of where the cursor is drawn    
    def getCursorRect(self):
        lw = 0
        if (len(self.lines) > 0):
            lw, lh = self.font.size( self.lines[ self.vpos ][ 0:self.hpos ] )
            
        r = pygame.Rect(lw, (self.vpos - self.vscroll) * self.line_h, self.cursor_w, self.line_h)
        return r
    
    # This function sets the cursor position according to an x/y value (such as by from a mouse click)
    def setCursorByXY(self, pos):
        (x, y) = pos
        self.vpos = ((int) (y / self.line_h)) + self.vscroll
        if (self.vpos >= len(self.lines)):
            self.vpos = len(self.lines) - 1
            
        currentLine = self.lines[ self.vpos ]
        
        for cnt in range(0, len(currentLine) ):
            self.hpos = cnt
            lw, lh = self.font.size( currentLine[ 0:self.hpos + 1 ] )
            if (lw > x):
                break
        
        lw, lh = self.font.size( currentLine )
        if (lw < x):
            self.hpos = len(currentLine)
            
        self.setCursorByHVPos()
        
    # This function sets the cursor position by the horizontal/vertical cursor position.    
    def setCursorByHVPos(self):
        line_cnt = 0
        char_cnt = 0
        
        for line in self.lines:
            line_char_start = char_cnt # The number of characters at the start of the line
            
            # Keep track of the character count for words
            char_cnt += len(line)

            # If we're on the proper line
            if (line_cnt == self.vpos):
                # Make sure that we're not trying to go over the edge of the current line
                if ( self.hpos > len(line) ):
                    self.hpos = len(line) - 1
                # Set the cursor position
                self.pos = line_char_start + self.hpos
                break    # Now that we've set our cursor position, we exit the loop
                
            line_cnt += 1        
    
    # Splits up the text found in the control's value, and assigns it into the lines array
    def doLines(self, max_line_w):
        self.line_h = 10
        self.lines = [] # Create an empty starter list to start things out.
        
        inx = 0
        line_start = 0
        while inx >= 0:
            # Find the next breakable whitespace
            # HACK: Find a better way to do this to include tabs and system characters and whatnot.
            prev_word_start = inx # Store the previous whitespace
            spc_inx = self.value.find(' ', inx+1)
            nl_inx = self.value.find('\n', inx+1)
            
            if (min(spc_inx, nl_inx) == -1):
                inx = max(spc_inx, nl_inx)
            else:
                inx = min(spc_inx, nl_inx)
                
            # Measure the current line
            lw, self.line_h = self.font.size( self.value[ line_start : inx ] )
            
            # If we exceeded the max line width, then create a new line
            if (lw > max_line_w):
                #Fall back to the previous word start
                self.lines.append(self.value[ line_start : prev_word_start + 1 ])
                line_start = prev_word_start + 1
                # TODO: Check for extra-long words here that exceed the length of a line, to wrap mid-word
                
            # If we reached the end of our text
            if (inx < 0):
                # Then make sure we added the last of the line
                if (line_start < len( self.value ) ):
                    self.lines.append( self.value[ line_start : len( self.value ) ] )
                else:
                    self.lines.append('')
            # If we reached a hard line break
            elif (self.value[inx] == "\n"):
                # Then make a line break here as well.
                newline = self.value[ line_start : inx + 1 ]
                newline = newline.replace("\n", " ") # HACK: We know we have a newline character, which doesn't print nicely, so make it into a space. Comment this out to see what I mean.
                self.lines.append( newline )
                
                line_start = inx + 1
            else:
                # Otherwise, we just continue progressing to the next space
                pass
        
    def _setvalue(self,v):
        self.__dict__['value'] = v
        self.send(CHANGE)
    
    def event(self,e):
        used = None
        if e.type == KEYDOWN:    
            used = True
            if e.key == K_BACKSPACE:
                if self.pos:
                    self._setvalue(self.value[:self.pos-1] + self.value[self.pos:])
                    self.pos -= 1
            elif e.key == K_DELETE:
                if len(self.value) > self.pos:
                    self._setvalue(self.value[:self.pos] + self.value[self.pos+1:])
            elif e.key == K_HOME: 
                # Find the previous newline
                newPos = self.value.rfind('\n', 0, self.pos)
                if (newPos >= 0):
                    self.pos = newPos
            elif e.key == K_END:
                # Find the previous newline
                newPos = self.value.find('\n', self.pos, len(self.value) )
                if (newPos >= 0):
                    self.pos = newPos
            elif e.key == K_LEFT:
                if self.pos > 0: self.pos -= 1
#                used = True
            elif e.key == K_RIGHT:
                if self.pos < len(self.value): self.pos += 1
#                used = True
            elif e.key == K_UP:
                self.vpos -= 1
                self.setCursorByHVPos()
            elif e.key == K_DOWN:
                self.vpos += 1
                self.setCursorByHVPos()
            # The following return/tab keys are standard for PGU widgets, but I took them out here to facilitate multi-line text editing
#            elif e.key == K_RETURN:
#                self.next()
#            elif e.key == K_TAB:
#                pass                
            else:
                #c = str(e.unicode)
                used = None
                try:
                    if (e.key == K_RETURN):
                        c = "\n"
                    elif (e.key == K_TAB):
                        c = "  "
                    else:
                        c = (e.unicode).encode('latin-1')
                    if c:
                        used = True
                        self._setvalue(self.value[:self.pos] + c + self.value[self.pos:])
                        self.pos += len(c)
                except: #ignore weird characters
                    pass
            self.repaint()
        elif e.type == MOUSEBUTTONDOWN:
            self.setCursorByXY(e.pos)
            self.repaint()
            
        elif e.type == FOCUS:
            self.repaint()
        elif e.type == BLUR:
            self.repaint()
        
        self.pcls = ""
        if self.container.myfocus is self: self.pcls = "focus"
        
        return used
    
    def __setattr__(self,k,v):
        if k == 'value':
            if v == None: v = ''
            v = str(v)
            self.pos = len(v)
        _v = self.__dict__.get(k,NOATTR)
        self.__dict__[k]=v
        if k == 'value' and _v != NOATTR and _v != v: 
            self.send(CHANGE)
            self.repaint()
            
# The first version of this code was done by Clint Herron, and is a modified version of input.py (by Phil Hassey).
# It is under the same license as the rest of the PGU library.




"""This modules defines the Widget class, which is the base of the PGU widget
hierarchy."""

import pygame

from . import pguglobals
from . import style

class SignalCallback:
    # The function to call
    func = None
    # The parameters to pass to the function (as a list)
    params = None

class Widget(object):
    """Base class for all PGU graphical objects.
        
    Example - Creating your own Widget:

        class Draw(gui.Widget):
            def paint(self,s):
                # Paint the pygame.Surface
                return
            
            def update(self,s):
                # Update the pygame.Surface and return the update rects
                return [pygame.Rect(0,0,self.rect.w,self.rect.h)]
                
            def event(self,e):
                # Handle the pygame.Event
                return
                
            def resize(self,width=None,height=None):
                # Return the width and height of this widget
                return 256,256
    """

    # The name of the widget (or None if not defined)
    name = None
    # The container this widget belongs to
    container = None
    # Whether this widget has been painted yet
    _painted = False
    # The widget used to paint the background
    background = None
    # ...
    _rect_content = None
    # A dictionary of signal callbacks, hashed by signal ID
    connects = None
    
    def __init__(self, **params): 
        """Create a new Widget instance given the style parameters.

        Keyword arguments:
            decorate -- whether to call theme.decorate(self) to allow the 
                theme a chance to decorate the widget. (default is true)
            style -- a dict of style parameters.
            x, y -- position parameters
            width, height -- size parameters
            align, valign -- alignment parameters, passed along to style
            font -- the font to use with this widget
            color -- the color property, if applicable
            background -- the widget used to paint the background
            cls -- class name as used by Theme
            name -- name of widget as used by Form.  If set, will call 
                form.add(self,name) to add the widget to the most recently 
                created Form.
            focusable -- True if this widget can receive focus via Tab, etc.
                (default is True)
            disabled -- True of this widget is disabled (defaults is False)
            value -- initial value

        """
        #object.Object.__init__(self) 
        self.connects = {}
        params.setdefault('decorate',True)
        params.setdefault('style',{})
        params.setdefault('focusable',True)
        params.setdefault('disabled',False)
        
        self.focusable = params['focusable']
        self.disabled = params['disabled']
        
        self.rect = pygame.Rect(params.get('x',0),
                                params.get('y',0),
                                params.get('width',0),
                                params.get('height',0))
        
        s = params['style']
        #some of this is a bit "theme-ish" but it is very handy, so these
        #things don't have to be put directly into the style.
        for att in ('align','valign','x','y','width','height','color','font','background'):
            if att in params: s[att] = params[att]
        self.style = style.Style(self,s)
        
        self.cls = 'default'
        if 'cls' in params: self.cls = params['cls']
        if 'name' in params:    
            from . import form
            self.name = params['name']
            if form.Form.form:
                form.Form.form.add(self)
                self.form = form.Form.form
        if 'value' in params: self.value = params['value']
        self.pcls = ""
        
        if params['decorate'] != False:
            if (not pguglobals.app):
                # TODO - fix this somehow
                from . import app
                app.App()
            pguglobals.app.theme.decorate(self,params['decorate'])

    def focus(self):
        """Focus this Widget."""
        if self.container: 
            if self.container.myfocus != self:  ## by Gal Koren
                self.container.focus(self)

    def blur(self): 
        """Blur this Widget."""
        if self.container: self.container.blur(self)

    def open(self):
        """Open this widget as a modal dialog."""
        #if getattr(self,'container',None) != None: self.container.open(self)
        pguglobals.app.open(self)

    def close(self, w=None):
        """Close this widget, if it is currently an open dialog."""
        #if getattr(self,'container',None) != None: self.container.close(self)
        if (not w):
            w = self
        pguglobals.app.close(w)

    def is_open(self):
        return (self in pguglobals.app.windows)

    def is_hovering(self):
        """Returns true if the mouse is hovering over this widget."""
        if self.container:
            return (self.container.myhover is self)
        return False

    def resize(self,width=None,height=None):
        """Resize this widget and all sub-widgets, returning the new size.

        This should be implemented by a subclass.

        """
        return (self.style.width, self.style.height)

    def chsize(self):
        """Signal that this widget has changed its size."""
        
        if (not self._painted): 
            return
        
        if (not self.container): 
            return
        
        if (pguglobals.app):
            pguglobals.app.chsize()

    def update(self,s):
        """Updates the surface and returns a rect list of updated areas

        This should be implemented by a subclass.

        """
        return
        
    def paint(self,s):
        """Render this widget onto the given surface

        This should be implemented by a subclass.

        """
        return

    def repaint(self): 
        """Request a repaint of this Widget."""
        if self.container: self.container.repaint(self)

    def repaintall(self):
        """Request a repaint of all Widgets."""
        if self.container: self.container.repaintall()

    def reupdate(self): 
        """Request a reupdate of this Widget."""
        if self.container: self.container.reupdate(self)

    def next(self): 
        """Pass focus to next Widget.
        
        Widget order determined by the order they were added to their container.

        """
        if self.container: self.container.next(self)

    def previous(self): 
        """Pass focus to previous Widget.
        
        Widget order determined by the order they were added to their container.

        """
        
        if self.container: self.container.previous(self)
    
    def get_abs_rect(self):
        """Returns the absolute rect of this widget on the App screen."""
        x, y = self.rect.x, self.rect.y
        cnt = self.container
        while cnt:
            x += cnt.rect.x
            y += cnt.rect.y
            if cnt._rect_content:
                x += cnt._rect_content.x
                y += cnt._rect_content.y
            cnt = cnt.container
        return pygame.Rect(x, y, self.rect.w, self.rect.h)

    def connect(self,code,func,*params):
        """Connect an event code to a callback function.
        
        Note that there may be multiple callbacks per event code.

        Arguments:
            code -- event type
            fnc -- callback function
            *values -- values to pass to callback.  

        Please note that callbacks may also have "magicaly" parameters.  
        Such as:

            _event -- receive the event
            _code -- receive the event code
            _widget -- receive the sending widget
        
        Example:
            def onclick(value):
                print 'click', value
            
            w = Button("PGU!")
            w.connect(gui.CLICK,onclick,'PGU Button Clicked')

        """
        if (not code in self.connects):
            self.connects[code] = []
        for cb in self.connects[code]:
            if (cb.func == func):
                # Already connected to this callback function
                return
        # Wrap the callback function and add it to the list
        cb = SignalCallback()
        cb.func = func
        cb.params = params
        self.connects[code].append(cb)

    # Remove signal handlers from the given event code. If func is specified,
    # only those handlers will be removed. If func is None, all handlers
    # will be removed.
    def disconnect(self, code, func=None):
        if (not code in self.connects):
            return
        if (not func):
            # Remove all signal handlers
            del self.connects[code]
        else:
            # Remove handlers that call 'func'
            n = 0
            callbacks = self.connects[code]
            while (n < len(callbacks)):
                if (callbacks[n].func == func):
                    # Remove this callback
                    del callbacks[n]
                else:
                    n += 1

    def send(self,code,event=None):
        """Send a code, event callback trigger."""
        if (not code in self.connects):
            return
        # Trigger all connected signal handlers
        for cb in self.connects[code]:
            func = cb.func
            values = list(cb.params)

            # Attempt to be compatible with previous versions of python
            try:
                code = func.__code__
            except:
                code = func.func_code

            nargs = code.co_argcount
            names = list(code.co_varnames)[:nargs]

            # If the function is bound to an instance, remove the first argument name. Again
            # we keep compatibility with older versions of python.
            if (hasattr(func, "__self__") and hasattr(func.__self__, "__class__") or 
                hasattr(func,'im_class')): 
                names.pop(0)
            
            args = []
            magic = {'_event':event,'_code':code,'_widget':self}
            for name in names:
                if name in magic.keys():
                    args.append(magic[name])
                elif len(values):
                    args.append(values.pop(0))
                else:
                    break
            args.extend(values)
            func(*args)
    
    def _event(self,e):
        if self.disabled: return
        self.send(e.type,e)
        return self.event(e)
        
    def event(self,e):
        """Called when an event is passed to this object.
        
        Please note that if you use an event, returning the value True
        will stop parent containers from also using the event.  (For example, if
        your widget handles TABs or arrow keys, and you don't want those to 
        also alter the focus.)

        This should be implemented by a subclass.

        """
        return

    def get_toplevel(self):
        """Returns the top-level widget (usually the Desktop) by following the
        chain of 'container' references."""
        top = self
        while (top.container):
            top = top.container
        return top

    def collidepoint(self, pos):
        """Test if the given point hits this widget. Over-ride this function
        for more advanced collision testing."""
        return self.rect.collidepoint(pos)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

"""
A simple, minimal Pygame-based backend.
It will only draw and support very basic keyboard input (ESC to quit).

There are no main dependencies other than the actual test you are running.
Note that this only relies on framework.py for the loading of this backend,
and holding the Keys class. If you write a test that depends only on this
backend, you can remove references to that file here and import this module
directly in your test.

To use this backend, try:
 % python -m examples.web --backend simple

NOTE: Examples with Step() re-implemented are not yet supported, as I wanted
to do away with the Settings class. This means the following will definitely
not work: Breakable, Liquid, Raycast, TimeOfImpact, ... (incomplete)
"""

import pygame
from pygame.locals import (QUIT, KEYDOWN, KEYUP)
import Box2D
from Box2D.b2 import (world, staticBody, dynamicBody, kinematicBody,
                      polygonShape, circleShape, edgeShape, loopShape)

TARGET_FPS = 60
PPM = 10.0
TIMESTEP = 1.0 / TARGET_FPS
VEL_ITERS, POS_ITERS = 10, 10
SCREEN_WIDTH, SCREEN_HEIGHT = 800, 600
SCREEN_OFFSETX, SCREEN_OFFSETY = SCREEN_WIDTH * 1.0 / 2.0, SCREEN_HEIGHT * 2.0 / 3
colors = {
    staticBody: (255, 255, 255, 255),
    dynamicBody: (127, 127, 127, 255),
    kinematicBody: (127, 127, 230, 255),
}


def fix_vertices(vertices):
    return [(int(SCREEN_OFFSETX + v[0]), int(SCREEN_OFFSETY - v[1])) for v in vertices]


def _draw_polygon(polygon, screen, body, fixture):
    transform = body.transform
    vertices = fix_vertices([transform * v * PPM for v in polygon.vertices])
    pygame.draw.polygon(
        screen, [c / 2.0 for c in colors[body.type]], vertices, 0)
    pygame.draw.polygon(screen, colors[body.type], vertices, 1)
polygonShape.draw = _draw_polygon


def _draw_circle(circle, screen, body, fixture):
    position = fix_vertices([body.transform * circle.pos * PPM])[0]
    pygame.draw.circle(screen, colors[body.type],
                       position, int(circle.radius * PPM))
circleShape.draw = _draw_circle


def _draw_edge(edge, screen, body, fixture):
    vertices = fix_vertices(
        [body.transform * edge.vertex1 * PPM, body.transform * edge.vertex2 * PPM])
    pygame.draw.line(screen, colors[body.type], vertices[0], vertices[1])
edgeShape.draw = _draw_edge


def _draw_loop(loop, screen, body, fixture):
    transform = body.transform
    vertices = fix_vertices([transform * v * PPM for v in loop.vertices])
    v1 = vertices[-1]
    for v2 in vertices:
        pygame.draw.line(screen, colors[body.type], v1, v2)
        v1 = v2
loopShape.draw = _draw_loop


def draw_world(screen, world):
    # Draw the world
    for body in world.bodies:
        for fixture in body.fixtures:
            fixture.shape.draw(screen, body, fixture)


class Keys(object):
    pass

# The following import is only needed to do the initial loading and
# overwrite the Keys class.
import framework
# Set up the keys (needed as the normal framework abstracts them between
# backends)
keys = [s for s in dir(pygame.locals) if s.startswith('K_')]
for key in keys:
    value = getattr(pygame.locals, key)
    setattr(Keys, key, value)
framework.Keys = Keys


class SimpleFramework(object):
    name = 'None'
    description = ''

    def __init__(self):
        self.world = world()

        print('Initializing pygame framework...')
        # Pygame Initialization
        pygame.init()
        caption = "Python Box2D Testbed - Simple backend - " + self.name
        pygame.display.set_caption(caption)

        # Screen and debug draw
        self.screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
        self.font = pygame.font.Font(None, 15)

        self.groundbody = self.world.CreateBody()

    def run(self):
        """
        Main loop.

        Updates the world and then the screen.
        """

        running = True
        clock = pygame.time.Clock()
        while running:
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN and event.key == Keys.K_ESCAPE):
                    running = False
                elif event.type == KEYDOWN:
                    self.Keyboard(event.key)
                elif event.type == KEYUP:
                    self.KeyboardUp(event.key)

            self.screen.fill((0, 0, 0))

            self.textLine = 15

            # Step the world
            self.world.Step(TIMESTEP, VEL_ITERS, POS_ITERS)
            self.world.ClearForces()

            draw_world(self.screen, self.world)

            # Draw the name of the test running
            self.Print(self.name, (127, 127, 255))

            if self.description:
                # Draw the name of the test running
                for s in self.description.split('\n'):
                    self.Print(s, (127, 255, 127))

            pygame.display.flip()
            clock.tick(TARGET_FPS)
            self.fps = clock.get_fps()

        self.world.contactListener = None
        self.world.destructionListener = None
        self.world.renderer = None

    def Print(self, str, color=(229, 153, 153, 255)):
        """
        Draw some text at the top status lines
        and advance to the next line.
        """
        self.screen.blit(self.font.render(
            str, True, color), (5, self.textLine))
        self.textLine += 15

    def Keyboard(self, key):
        """
        Callback indicating 'key' has been pressed down.
        The keys are mapped after pygame's style.

         from .framework import Keys
         if key == Keys.K_z:
             ...
        """
        pass

    def KeyboardUp(self, key):
        """
        Callback indicating 'key' has been released.
        See Keyboard() for key information
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Based on Chris Campbell's tutorial from iforce2d.net:
http://www.iforce2d.net/b2dtut/top-down-car
"""

from Box2D.examples.framework import (Framework, Keys, main)
import math


class TDGroundArea(object):
    """
    An area on the ground that the car can run over
    """

    def __init__(self, friction_modifier):
        self.friction_modifier = friction_modifier


class TDTire(object):

    def __init__(self, car, max_forward_speed=100.0,
                 max_backward_speed=-20, max_drive_force=150,
                 turn_torque=15, max_lateral_impulse=3,
                 dimensions=(0.5, 1.25), density=1.0,
                 position=(0, 0)):

        world = car.body.world

        self.current_traction = 1
        self.turn_torque = turn_torque
        self.max_forward_speed = max_forward_speed
        self.max_backward_speed = max_backward_speed
        self.max_drive_force = max_drive_force
        self.max_lateral_impulse = max_lateral_impulse
        self.ground_areas = []

        self.body = world.CreateDynamicBody(position=position)
        self.body.CreatePolygonFixture(box=dimensions, density=density)
        self.body.userData = {'obj': self}

    @property
    def forward_velocity(self):
        body = self.body
        current_normal = body.GetWorldVector((0, 1))
        return current_normal.dot(body.linearVelocity) * current_normal

    @property
    def lateral_velocity(self):
        body = self.body

        right_normal = body.GetWorldVector((1, 0))
        return right_normal.dot(body.linearVelocity) * right_normal

    def update_friction(self):
        impulse = -self.lateral_velocity * self.body.mass
        if impulse.length > self.max_lateral_impulse:
            impulse *= self.max_lateral_impulse / impulse.length

        self.body.ApplyLinearImpulse(self.current_traction * impulse,
                                     self.body.worldCenter, True)

        aimp = 0.1 * self.current_traction * \
            self.body.inertia * -self.body.angularVelocity
        self.body.ApplyAngularImpulse(aimp, True)

        current_forward_normal = self.forward_velocity
        current_forward_speed = current_forward_normal.Normalize()

        drag_force_magnitude = -2 * current_forward_speed
        self.body.ApplyForce(self.current_traction * drag_force_magnitude * current_forward_normal,
                             self.body.worldCenter, True)

    def update_drive(self, keys):
        if 'up' in keys:
            desired_speed = self.max_forward_speed
        elif 'down' in keys:
            desired_speed = self.max_backward_speed
        else:
            return

        # find the current speed in the forward direction
        current_forward_normal = self.body.GetWorldVector((0, 1))
        current_speed = self.forward_velocity.dot(current_forward_normal)

        # apply necessary force
        force = 0.0
        if desired_speed > current_speed:
            force = self.max_drive_force
        elif desired_speed < current_speed:
            force = -self.max_drive_force
        else:
            return

        self.body.ApplyForce(self.current_traction * force * current_forward_normal,
                             self.body.worldCenter, True)

    def update_turn(self, keys):
        if 'left' in keys:
            desired_torque = self.turn_torque
        elif 'right' in keys:
            desired_torque = -self.turn_torque
        else:
            return

        self.body.ApplyTorque(desired_torque, True)

    def add_ground_area(self, ud):
        if ud not in self.ground_areas:
            self.ground_areas.append(ud)
            self.update_traction()

    def remove_ground_area(self, ud):
        if ud in self.ground_areas:
            self.ground_areas.remove(ud)
            self.update_traction()

    def update_traction(self):
        if not self.ground_areas:
            self.current_traction = 1
        else:
            self.current_traction = 0
            mods = [ga.friction_modifier for ga in self.ground_areas]

            max_mod = max(mods)
            if max_mod > self.current_traction:
                self.current_traction = max_mod


class TDCar(object):
    vertices = [(1.5, 0.0),
                (3.0, 2.5),
                (2.8, 5.5),
                (1.0, 10.0),
                (-1.0, 10.0),
                (-2.8, 5.5),
                (-3.0, 2.5),
                (-1.5, 0.0),
                ]

    tire_anchors = [(-3.0, 0.75),
                    (3.0, 0.75),
                    (-3.0, 8.50),
                    (3.0, 8.50),
                    ]

    def __init__(self, world, vertices=None,
                 tire_anchors=None, density=0.1, position=(0, 0),
                 **tire_kws):
        if vertices is None:
            vertices = TDCar.vertices

        self.body = world.CreateDynamicBody(position=position)
        self.body.CreatePolygonFixture(vertices=vertices, density=density)
        self.body.userData = {'obj': self}

        self.tires = [TDTire(self, **tire_kws) for i in range(4)]

        if tire_anchors is None:
            anchors = TDCar.tire_anchors

        joints = self.joints = []
        for tire, anchor in zip(self.tires, anchors):
            j = world.CreateRevoluteJoint(bodyA=self.body,
                                          bodyB=tire.body,
                                          localAnchorA=anchor,
                                          # center of tire
                                          localAnchorB=(0, 0),
                                          enableMotor=False,
                                          maxMotorTorque=1000,
                                          enableLimit=True,
                                          lowerAngle=0,
                                          upperAngle=0,
                                          )

            tire.body.position = self.body.worldCenter + anchor
            joints.append(j)

    def update(self, keys, hz):
        for tire in self.tires:
            tire.update_friction()

        for tire in self.tires:
            tire.update_drive(keys)

        # control steering
        lock_angle = math.radians(40.)
        # from lock to lock in 0.5 sec
        turn_speed_per_sec = math.radians(160.)
        turn_per_timestep = turn_speed_per_sec / hz
        desired_angle = 0.0

        if 'left' in keys:
            desired_angle = lock_angle
        elif 'right' in keys:
            desired_angle = -lock_angle

        front_left_joint, front_right_joint = self.joints[2:4]
        angle_now = front_left_joint.angle
        angle_to_turn = desired_angle - angle_now

        # TODO fix b2Clamp for non-b2Vec2 types
        if angle_to_turn < -turn_per_timestep:
            angle_to_turn = -turn_per_timestep
        elif angle_to_turn > turn_per_timestep:
            angle_to_turn = turn_per_timestep

        new_angle = angle_now + angle_to_turn
        # Rotate the tires by locking the limits:
        front_left_joint.SetLimits(new_angle, new_angle)
        front_right_joint.SetLimits(new_angle, new_angle)


class TopDownCar (Framework):
    name = "Top Down Car"
    description = "Keys: accel = w, reverse = s, left = a, right = d"

    def __init__(self):
        super(TopDownCar, self).__init__()
        # Top-down -- no gravity in the screen plane
        self.world.gravity = (0, 0)

        self.key_map = {Keys.K_w: 'up',
                        Keys.K_s: 'down',
                        Keys.K_a: 'left',
                        Keys.K_d: 'right',
                        }

        # Keep track of the pressed keys
        self.pressed_keys = set()

        # The walls
        boundary = self.world.CreateStaticBody(position=(0, 20))
        boundary.CreateEdgeChain([(-30, -30),
                                  (-30, 30),
                                  (30, 30),
                                  (30, -30),
                                  (-30, -30)]
                                 )

        # A couple regions of differing traction
        self.car = TDCar(self.world)
        gnd1 = self.world.CreateStaticBody(userData={'obj': TDGroundArea(0.5)})
        fixture = gnd1.CreatePolygonFixture(
            box=(9, 7, (-10, 15), math.radians(20)))
        # Set as sensors so that the car doesn't collide
        fixture.sensor = True

        gnd2 = self.world.CreateStaticBody(userData={'obj': TDGroundArea(0.2)})
        fixture = gnd2.CreatePolygonFixture(
            box=(9, 5, (5, 20), math.radians(-40)))
        fixture.sensor = True

    def Keyboard(self, key):
        key_map = self.key_map
        if key in key_map:
            self.pressed_keys.add(key_map[key])
        else:
            super(TopDownCar, self).Keyboard(key)

    def KeyboardUp(self, key):
        key_map = self.key_map
        if key in key_map:
            self.pressed_keys.remove(key_map[key])
        else:
            super(TopDownCar, self).KeyboardUp(key)

    def handle_contact(self, contact, began):
        # A contact happened -- see if a wheel hit a
        # ground area
        fixture_a = contact.fixtureA
        fixture_b = contact.fixtureB

        body_a, body_b = fixture_a.body, fixture_b.body
        ud_a, ud_b = body_a.userData, body_b.userData
        if not ud_a or not ud_b:
            return

        tire = None
        ground_area = None
        for ud in (ud_a, ud_b):
            obj = ud['obj']
            if isinstance(obj, TDTire):
                tire = obj
            elif isinstance(obj, TDGroundArea):
                ground_area = obj

        if ground_area is not None and tire is not None:
            if began:
                tire.add_ground_area(ground_area)
            else:
                tire.remove_ground_area(ground_area)

    def BeginContact(self, contact):
        self.handle_contact(contact, True)

    def EndContact(self, contact):
        self.handle_contact(contact, False)

    def Step(self, settings):
        self.car.update(self.pressed_keys, settings.hz)

        super(TopDownCar, self).Step(settings)

        tractions = [tire.current_traction for tire in self.car.tires]
        self.Print('Current tractions: %s' % tractions)

if __name__ == "__main__":
    main(TopDownCar)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Based on the paper:
    "Simulation of 2D physics of objects captured by web camera using OpenCV
    and Box2D"
    by Michal Sedlak (sedlak@stuba.sk)
    Presented at the 18th International Conference on Process Control, 6/14-17,
        2011
    Available here: http://www.kirp.chtf.stuba.sk/pc11/data/papers/053.pdf

If no webcam is detected, the image from the paper will be used.
Requires: pygame, OpenCV
pygame: www.pygame.org
OpenCV: opencv.willowgarage.com (simple installer for windows here
        http://www.lfd.uci.edu/~gohlke/pythonlibs/ )
"""

from __future__ import print_function
import cv
import Box2D as b2
from triangulate_seidel import Triangulator

THRESHOLD = 55
PPM = 20.0  # pixels per meter
TARGET_FPS = 60
TIME_STEP = 1.0 / TARGET_FPS
SCREEN_WIDTH, SCREEN_HEIGHT = 800, 600
SCREEN_OFFSETX, SCREEN_OFFSETY = SCREEN_WIDTH * 1.0 / 4.0, SCREEN_HEIGHT


class CVObject(object):

    def __init__(self, world, image=None):
        self.world = world
        self.contour_bodies = []
        self.x_scale = 30.0
        self.y_scale = 30.0
        self.object_bodies = {}
        self._max_recursion = 200
        if image is not None:
            self.bodies_from_image(image)

    def create_body(self, cont, h, v, density):
        cont_m = [(point[0] / self.x_scale, point[1] / self.y_scale)
                  for point in cont]

        if v == 0:
            body = self.world.CreateStaticBody(position=(0, 0))
            body.CreateEdgeChain(cont_m)
        elif v == 1:
            try:
                body = self.object_bodies[h]
            except:
                body = self.world.CreateDynamicBody(position=(0, 0))
                body.CreateEdgeChain(cont_m)

                t = Triangulator(cont_m)
                for poly in t.triangles():
                    # poly = ((x1, y1), (x2, y2), (x3, y3))
                    body.CreatePolygonFixture(vertices=poly, density=1.0,
                                              restitution=0.0, friction=0.0)

    def _objects_from_contours(self, recurs, cont, h=0, v=0, ret=None):
        if recurs > self._max_recursion:
            return

        if v > 0:
            density = 1.0
        else:
            density = 0.0

        if len(cont) > 8:
            body = self.create_body(cont, h, v, density)
            if ret is not None:
                ret.append(body)

        if cont.v_next():
            v += 1
            self._objects_from_contours(recurs + 1, cont.v_next(), h, v, ret)
            v -= 1

        if cont.h_next():
            h += 1
            self._objects_from_contours(recurs + 1, cont.h_next(), h, v, ret)
            h -= 1

    def objects_from_contours(self, cont):
        ret = []
        self._objects_from_contours(0, cont, 0, 0, ret)
        return ret

    def detect_outline(self, image, threshold=THRESHOLD):
        img_size = cv.GetSize(image)
        grayscale = cv.CreateImage(img_size, 8, 1)
        cv.CvtColor(image, grayscale, cv.CV_BGR2GRAY)
        cv.EqualizeHist(grayscale, grayscale)

        storage = cv.CreateMemStorage(0)
        cv.Threshold(grayscale, grayscale, threshold, 255, cv.CV_THRESH_BINARY)
        contours = cv.FindContours(grayscale, cv.CreateMemStorage(),
                                   cv.CV_RETR_TREE, cv.CV_CHAIN_APPROX_SIMPLE)

        if len(contours) > 0:
            return cv.ApproxPoly(contours, storage, cv.CV_POLY_APPROX_DP, 1.5, 1)
        return contours

    def bodies_from_image(self, image, flip=True):
        if flip:
            flipped = cv.CreateImage(
                (image.width, image.height), image.depth, image.channels)
            cv.Flip(image, flipped, flipMode=0)
            image = flipped

        contours = self.detect_outline(image)
        return self.objects_from_contours(contours)

camera = None


def get_image():
    # Grab an image from the camera
    global camera
    try:
        # raise # <-- uncomment to force using the journal image
        camera = cv.CaptureFromCAM(-1)  # need to hold onto this instance
        image = cv.QueryFrame(camera)
        if image is None:
            raise Exception('Invalid image captured?')
    except Exception as ex:
        print('Unable to grab an image from the camera. (%s)' % ex)
        print('Using the journal image instead.')
        image = cv.LoadImage('journal_image.png')
    return image


def main():
    # Create the world
    world = b2.b2World(gravity=(0, -10), doSleep=True)

    # Get the OpenCV/Box2D object
    image = get_image()
    # Bring that image into Box2D
    cvo = CVObject(world, image)

    # ... and the rest is standard pygame visualization
    screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), 0, 32)
    pygame.display.set_caption('pybox2d/OpenCV example')
    clock = pygame.time.Clock()

    # And a static body to hold the ground shape
    world.CreateStaticBody(
        position=(0, 0),
        shapes=b2.b2PolygonShape(box=(50, 1)),
    )

    colors = {
        b2.b2_staticBody: (255, 255, 255, 255),
        b2.b2_dynamicBody: (127, 127, 127, 255),
    }

    def fix_vertices(vertices):
        return [(int(SCREEN_OFFSETX + v[0]), int(SCREEN_OFFSETY - v[1]))
                for v in vertices]

    def _draw_polygon(polygon, screen, body, fixture):
        transform = body.transform
        vertices = fix_vertices(
            [transform * v * PPM for v in polygon.vertices])
        pygame.draw.polygon(screen,
                            [c / 2.0 for c in colors[body.type]],
                            vertices, 0)
        pygame.draw.polygon(screen, colors[body.type], vertices, 1)
    b2.b2PolygonShape.draw = _draw_polygon

    def _draw_circle(circle, screen, body, fixture):
        position = fix_vertices([body.transform * circle.pos * PPM])[0]
        pygame.draw.circle(screen, colors[body.type], position,
                           int(circle.radius * PPM))
    b2.b2CircleShape.draw = _draw_circle

    def _draw_edge(edge, screen, body, fixture):
        vertices = fix_vertices([body.transform * edge.vertex1 * PPM,
                                 body.transform * edge.vertex2 * PPM])
        pygame.draw.line(screen, colors[body.type], vertices[0], vertices[1])
    b2.b2EdgeShape.draw = _draw_edge

    display_image = True
    if display_image:
        # opencv image -> pygame image
        image_rgb = cv.CreateMat(image.height, image.width, cv.CV_8UC3)
        cv.CvtColor(image, image_rgb, cv.CV_BGR2RGB)
        pg_img = pygame.image.frombuffer(
            image_rgb.tostring(), cv.GetSize(image_rgb), "RGB")
        pg_img.set_alpha(100)

    running = True
    while running:
        # Check the event queue
        for event in pygame.event.get():
            if (event.type == QUIT
                    or (event.type == KEYDOWN and event.key == K_ESCAPE)):
                # The user closed the window or pressed escape
                running = False

        screen.fill((0, 0, 0, 0))
        if display_image:
            screen.blit(pg_img, (0, 0))

        # Draw the world
        for body in world.bodies:
            for fixture in body.fixtures:
                fixture.shape.draw(screen, body, fixture)

        # Make Box2D simulate the physics of our world for one step.
        world.Step(TIME_STEP, 10, 10)

        # Flip the screen and try to keep at the target FPS
        pygame.display.flip()
        clock.tick(TARGET_FPS)

    pygame.quit()
    print('Done!')


if __name__ == '__main__':
    import pygame
    from pygame.locals import (QUIT, KEYDOWN, K_ESCAPE)
    main()



# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2015 John Stowers
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

"""
Global Keys:
    Space  - shoot projectile
    Z/X    - zoom
    Escape - quit

Other keys can be set by the individual test.

Mouse:
    Left click  - select/drag body (creates mouse joint)

"""
import string
import time

import cv2
import numpy as np

import framework
from framework import FrameworkBase, fwSettings

from Box2D import b2DrawExtended, b2Vec2

from opencv_draw import cvcolor, cvcoord


class OpencvDraw(framework.b2DrawExtended):
    """
    This debug draw class accepts callbacks from Box2D (which specifies what to
    draw) and handles all of the rendering.

    If you are writing your own game, you likely will not want to use debug
    drawing.  Debug drawing, as its name implies, is for debugging.
    """
    surface = None
    axisScale = 10.0

    def __init__(self, **kwargs):
        b2DrawExtended.__init__(self, **kwargs)
        self.flipX = False
        self.flipY = True
        self.convertVertices = True

    def StartDraw(self):
        self.zoom = self.test.viewZoom
        self.center = self.test.viewCenter
        self.offset = self.test.viewOffset
        self.screenSize = self.test.screenSize

    def EndDraw(self):
        pass

    def DrawPoint(self, p, size, color):
        """
        Draw a single point at point p given a pixel size and color.
        """
        self.DrawCircle(p, size / self.zoom, color, drawwidth=0)

    def DrawAABB(self, aabb, color):
        """
        Draw a wireframe around the AABB with the given color.
        """
        points = [(aabb.lowerBound.x, aabb.lowerBound.y),
                  (aabb.upperBound.x, aabb.lowerBound.y),
                  (aabb.upperBound.x, aabb.upperBound.y),
                  (aabb.lowerBound.x, aabb.upperBound.y)]

        pts = np.array(points, np.int32)
        pts = pts.reshape((-1, 1, 2))
        cv2.polylines(self.surface, [pts], True, cvcolor(color))

    def DrawSegment(self, p1, p2, color):
        """
        Draw the line segment from p1-p2 with the specified color.
        """
        cv2.line(self.surface, cvcoord(p1), cvcoord(p2), cvcolor(color), 1)

    def DrawTransform(self, xf):
        """
        Draw the transform xf on the screen
        """
        p1 = xf.position
        p2 = self.to_screen(p1 + self.axisScale * xf.R.x_axis)
        p3 = self.to_screen(p1 + self.axisScale * xf.R.y_axis)
        p1 = self.to_screen(p1)
        cv2.line(self.surface, cvcoord(p1), cvcoord(p2), (0, 0, 255), 1)
        cv2.line(self.surface, cvcoord(p1), cvcoord(p3), (0, 255, 0), 1)

    def DrawCircle(self, center, radius, color, drawwidth=1):
        """
        Draw a wireframe circle given the center, radius, axis of orientation
        and color.
        """
        radius *= self.zoom
        if radius < 1:
            radius = 1
        else:
            radius = int(radius)

        cv2.circle(self.surface, cvcoord(center),
                   radius, cvcolor(color), drawwidth)

    def DrawSolidCircle(self, center, radius, axis, color):
        """
        Draw a solid circle given the center, radius, axis of orientation and
        color.
        """
        radius *= self.zoom
        if radius < 1:
            radius = 1
        else:
            radius = int(radius)

        FILL = False

        cv2.circle(self.surface, cvcoord(center), radius,
                   cvcolor(color), -1 if FILL else 1)

        cv2.line(self.surface, cvcoord(center),
                 cvcoord((center[0] - radius * axis[0],
                          center[1] + radius * axis[1])),
                 (0, 0, 255),
                 1)

    def DrawPolygon(self, vertices, color):
        """
        Draw a wireframe polygon given the screen vertices with the specified
        color.
        """
        if not vertices:
            return

        if len(vertices) == 2:
            cv2.line(self.surface, cvcoord(vertices[0]), cvcoord(
                vertices[1]), cvcolor(color), 1)
        else:
            pts = np.array(vertices, np.int32)
            pts = pts.reshape((-1, 1, 2))
            cv2.polylines(self.surface, [pts], True, cvcolor(color))

    def DrawSolidPolygon(self, vertices, color):
        """
        Draw a filled polygon given the screen vertices with the specified color.
        """
        FILL = False

        if not FILL:
            self.DrawPolygon(vertices, color)
            return

        if not vertices:
            return

        if len(vertices) == 2:
            cv2.line(self.surface, cvcoord(vertices[0]), cvcoord(
                vertices[1]), cvcolor(color), 1)
        else:
            pts = np.array(vertices, np.int32)
            pts = pts.reshape((-1, 1, 2))
            cv2.fillPoly(self.surface, [pts], cvcolor(color))

# Only support ascii keys
# The following import is only needed to do the initial loading and
# overwrite the Keys class.


class OpencvKeysType(type):
    def __getattr__(cls, key):
        return getattr(cls, key, None)


class OpencvKeys:
    __metaclass__ = OpencvKeysType


for key in string.ascii_lowercase + string.digits:
    setattr(OpencvKeys, 'K_%c' % key, ord(key))


framework.Keys = OpencvKeys


class OpencvFramework(FrameworkBase):

    def __init__(self, w=640, h=480, resizable=False):
        super(OpencvFramework, self).__init__()

        if fwSettings.onlyInit:  # testing mode doesn't initialize opencv
            return

        self._viewZoom = 10.0
        self._viewCenter = None
        self._viewOffset = None
        self.screenSize = None
        self.fps = 0

        self.screen = np.zeros((h, w, 3), np.uint8)

        if resizable:
            cv2.namedWindow(self.name, getattr(cv2, 'WINDOW_NORMAL', 0))
            cv2.resizeWindow(self.name, w, h)
        else:
            cv2.namedWindow(self.name, getattr(cv2, 'WINDOW_AUTOSIZE', 1))

        cv2.setMouseCallback(self.name, self._on_mouse)

        self._t0 = time.time()

        self.textLine = 30
        self._font_name = cv2.FONT_HERSHEY_SIMPLEX
        self._font_scale = 0.5
        self._font_thickness = 1

        (_, self._font_h), _ = cv2.getTextSize("X", self._font_name,
                                               self._font_scale,
                                               self._font_thickness)

        self.screenSize = b2Vec2(w, h)

        self.renderer = OpencvDraw(surface=self.screen, test=self)
        self.world.renderer = self.renderer

        self.viewCenter = (0, 20.0)
        self.groundbody = self.world.CreateBody()

    # mouse callback function
    def _on_mouse(self, event, x, y, flags, param):
        p = self.ConvertScreenToWorld(x, y)
        if event == cv2.EVENT_LBUTTONDOWN:
            self.MouseDown(p)
        if event == cv2.EVENT_LBUTTONUP:
            self.MouseUp(p)
        elif event == cv2.EVENT_MOUSEMOVE:
            self.MouseMove(p)

    def setCenter(self, value):
        """
        Updates the view offset based on the center of the screen.

        Tells the debug draw to update its values also.
        """
        self._viewCenter = b2Vec2(*value)
        self._viewCenter *= self._viewZoom
        self._viewOffset = self._viewCenter - self.screenSize / 2

    def setZoom(self, zoom):
        self._viewZoom = zoom

    viewZoom = property(lambda self: self._viewZoom, setZoom,
                        doc='Zoom factor for the display')
    viewCenter = property(lambda self: self._viewCenter / self._viewZoom,
                          setCenter,
                          doc='Screen center in camera coordinates')
    viewOffset = property(lambda self: self._viewOffset,
                          doc='The offset of the top-left corner of the screen')

    def run(self):
        """
        Main loop.

        Continues to run while checkEvents indicates the user has requested to
        quit.

        Updates the screen and tells the GUI to paint itself.
        """
        while True:
            self._t1 = time.time()

            dt = 1.0 / self.settings.hz
            key = 0xFF & cv2.waitKey(int(dt * 1000.0))
            if key == 27:
                break
            elif key != 255:
                if key == 32:               # Space
                    self.LaunchRandomBomb()
                elif key == 81:             # Left
                    self.viewCenter -= (0.5, 0)
                elif key == 83:             # Right
                    self.viewCenter += (0.5, 0)
                elif key == 82:             # Up
                    self.viewCenter += (0, 0.5)
                elif key == 84:             # Down
                    self.viewCenter -= (0, 0.5)
                elif key == 80:             # Home
                    self.viewZoom = 1.0
                    self.viewCenter = (0.0, 20.0)
                elif key == ord('z'):       # Zoom in
                    self.viewZoom = min(1.1 * self.viewZoom, 50.0)
                elif key == ord('x'):       # Zoom out
                    self.viewZoom = max(0.9 * self.viewZoom, 0.02)
                else:
                    self.Keyboard(key)

            self.screen.fill(0)
            self.SimulationLoop()
            cv2.imshow(self.name, self.screen)

            dt = self._t1 - self._t0
            self._t0 = self._t1
            self.fps = 1 / dt

        self.world.contactListener = None
        self.world.destructionListener = None
        self.world.renderer = None

    def ConvertScreenToWorld(self, x, y):
        x = (x + self.viewOffset.x) / self.viewZoom
        y = ((self.screenSize.y - y + self.viewOffset.y) / self.viewZoom)
        return b2Vec2(x, y)

    def DrawStringAt(self, x, y, str, color=(229, 153, 153, 255)):
        """
        Draw some text, str, at screen coordinates (x, y).
        """
        color = (color[2], color[1], color[0])
        cv2.putText(self.screen, str, (x, y),
                    self._font_name, self._font_scale, color,
                    self._font_thickness)

    def Print(self, str, color=(229, 153, 153, 255)):
        """
        Draw some text at the top status lines
        and advance to the next line.
        """
        color = (color[2], color[1], color[0])
        cv2.putText(self.screen, str, (5, self.textLine),
                    self._font_name, self._font_scale, color,
                    self._font_thickness)
        self.textLine += self._font_h + 2

    def Keyboard(self, key):
        """
        Callback indicating 'key' has been pressed down.
        The keys are mapped after pygame's style.

         from framework import Keys
         if key == Keys.K_z:
             ...
        """
        pass

    def KeyboardUp(self, key):
        """
        Callback indicating 'key' has been released.
        See Keyboard() for key information
        """
        pass



"""Contains the global reference to the PGU application."""

# pguglobals.py - A place to stick global variables that need to be accessed
#                 from other modules. To avoid problems with circular imports
#                 this module should not import any other PGU module.

# A global reference to the application instance (App class)
app = None




"""
"""

from . import pguglobals

class Style:
    """The class used by widget for the widget.style
    
    This object is used mainly as a dictionary, accessed via widget.style.attr, 
    as opposed to widget.style['attr'].  It automatically grabs information 
    from the theme via value = theme.get(widget.cls,widget.pcls,attr)

    """
    def __init__(self, obj, dict):
        self.obj = obj
        for k,v in dict.items(): self.__dict__[k]=v

    def __getattr__(self, attr):
        value = pguglobals.app.theme.get(self.obj.cls, self.obj.pcls, attr)

        if attr in (
            'border_top','border_right','border_bottom','border_left',
            'padding_top','padding_right','padding_bottom','padding_left',
            'margin_top','margin_right','margin_bottom','margin_left',
            'align','valign','width','height',
            ): self.__dict__[attr] = value
        return value

    def __setattr__(self, attr, value):
        self.__dict__[attr] = value




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2EdgeShape, b2PolygonShape)


class EdgeTest (Framework):
    name = "EdgeTest"
    description = "Utilizes b2EdgeShape"

    def __init__(self):
        super(EdgeTest, self).__init__()

        v1 = (-10.0, 0.0)
        v2 = (-7.0, -1.0)
        v3 = (-4.0, 0.0)
        v4 = (0.0, 0.0)
        v5 = (4.0, 0.0)
        v6 = (7.0, 1.0)
        v7 = (10.0, 0.0)

        shapes = [b2EdgeShape(vertices=[None, v1, v2, v3]),
                  b2EdgeShape(vertices=[v1, v2, v3, v4]),
                  b2EdgeShape(vertices=[v2, v3, v4, v5]),
                  b2EdgeShape(vertices=[v3, v4, v5, v6]),
                  b2EdgeShape(vertices=[v4, v5, v6, v7]),
                  b2EdgeShape(vertices=[v5, v6, v7])
                  ]
        ground = self.world.CreateStaticBody(shapes=shapes)

        box = self.world.CreateDynamicBody(
            position=(0.5, 0.6),
            allowSleep=False,
            shapes=b2PolygonShape(box=(0.5, 0.5))
        )

if __name__ == "__main__":
    main(EdgeTest)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import itertools
import sys

class testJoints (unittest.TestCase):
    world = None
    dbody1 = None
    dbody2 = None
    sbody1 = None
    sbody2 = None

    def _fail(self, s):
        # This little function would save us from an AssertionError on garbage collection.
        # It forces the gc to take care of the world early.
        self.world = None
        self.dbody1 = None
        self.dbody2 = None
        self.sbody1 = None
        self.sbody2 = None
        self.b2 = None
        self.fail(s)

    def setUp(self):
        self.b2 = __import__('Box2D')
        try:
            self.world = self.b2.b2World(self.b2.b2Vec2(0.0, -10.0), True)
        except Exception as ex:
            self.fail("Failed to create world (%s)" % ex)

        try:
            self.dbody1 = self.create_body((-3, 12))
            self.dbody1.userData = "dbody1"
            self.dbody2 = self.create_body((0, 12))
            self.dbody2.userData = "dbody2"
        except Exception as ex:
            self._fail("Failed to create dynamic bodies (%s)" % ex)

        try:
            self.sbody1 = self.create_body((0, 0), False)
            self.sbody1.userData = "sbody1"
            self.sbody2 = self.create_body((1, 4), False)
            self.sbody2.userData = "sbody2"
        except Exception as ex:
            self._fail("Failed to create static bodies (%s)" % ex)

    def create_body(self, position, dynamic=True):
        bodyDef = self.b2.b2BodyDef()
        fixtureDef = self.b2.b2FixtureDef()
        if dynamic:
            bodyDef.type = self.b2.b2_dynamicBody
            fixtureDef.density = 1
        else:
            bodyDef.type = self.b2.b2_staticBody
            fixtureDef.density = 0

        bodyDef.position = position
        body = self.world.CreateBody(bodyDef)

        dynamicBox = self.b2.b2PolygonShape()
        dynamicBox.SetAsBox(1, 1)

        fixtureDef.shape = dynamicBox
        fixtureDef.friction = 0.3

        body.CreateFixture(fixtureDef)
        return body

    def create_circle_body(self, position, dynamic=True):
        bodyDef = self.b2.b2BodyDef()
        fixtureDef = self.b2.b2FixtureDef()
        if dynamic:
            bodyDef.type = self.b2.b2_dynamicBody
            fixtureDef.density = 1
        else:
            bodyDef.type = self.b2.b2_staticBody
            fixtureDef.density = 0

        bodyDef.position = position
        body = self.world.CreateBody(bodyDef)

        body.CreateFixture(shape=self.b2.b2CircleShape(radius=1.0), density=1.0, friction=0.3)
        return body

    def step_world(self, steps=10):
        timeStep = 1.0 / 60
        vel_iters, pos_iters = 6, 2

        for i in range(steps):
            self.world.Step(timeStep, vel_iters, pos_iters)

    def check(self, dfn, joint, prop, joint_prop=""):
        a = getattr(dfn, prop)
        if joint_prop:
            b = getattr(joint, joint_prop)
        else:
            b = getattr(joint, prop)

        self.assertEquals(a, b, "Property not equal from definition to joint: %s (dfn %s != joint %s)" % (prop, a, b) )

    # ---- revolute joint ----
    def revolute_definition(self, body1, body2, anchor):
        dfn=self.b2.b2RevoluteJointDef()
        dfn.Initialize(body1, body2, anchor)
        dfn.motorSpeed = 1.0 * self.b2.b2_pi
        dfn.maxMotorTorque = 10000.0
        dfn.enableMotor = False
        dfn.lowerAngle = -0.25 * self.b2.b2_pi
        dfn.upperAngle = 0.5 * self.b2.b2_pi
        dfn.enableLimit = True
        dfn.collideConnected = True
        return dfn

    def revolute_asserts(self, dfn, joint):
        self.check(dfn, joint, "motorSpeed")
        self.check(dfn, joint, "lowerAngle", "lowerLimit")
        self.check(dfn, joint, "upperAngle", "upperLimit")
        self.check(dfn, joint, "enableMotor", "motorEnabled")
        self.check(dfn, joint, "enableLimit", "limitEnabled")
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")

    def revolute_checks(self, dfn, joint):
        # check to make sure they are at least accessible
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)
        joint.GetMotorTorque(1.0)
        i = joint.angle
        i = joint.speed
        i = joint.anchorA
        i = joint.anchorB
        joint.upperLimit = 2
        joint.maxMotorTorque = 10.0
        joint.foobar = 2

    # ---- prismatic joint ----
    def prismatic_definition(self, body1, body2, anchor, axis):
        dfn=self.b2.b2PrismaticJointDef()
        dfn.Initialize(body1, body2, anchor, axis)
        dfn.motorSpeed = 10
        dfn.maxMotorForce = 1000.0
        dfn.enableMotor = True
        dfn.lowerTranslation = 0
        dfn.upperTranslation = 20
        dfn.enableLimit = True
        return dfn

    def prismatic_asserts(self, dfn, joint):
        self.check(dfn, joint, "motorSpeed")
        self.check(dfn, joint, "lowerTranslation", "lowerLimit")
        self.check(dfn, joint, "upperTranslation", "upperLimit")
        self.check(dfn, joint, "enableMotor", "motorEnabled")
        self.check(dfn, joint, "enableLimit", "limitEnabled")
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")
        self.check(dfn, joint, "maxMotorForce")

    def prismatic_checks(self, dfn, joint):
        # check to make sure they are at least accessible
        i = joint.GetMotorForce(1.0)
        i = joint.anchorA
        i = joint.anchorB
        i = joint.speed
        i = joint.translation
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- distance joint ----
    def distance_definition(self, body1, body2, anchorA, anchorB):
        dfn=self.b2.b2DistanceJointDef()
        dfn.Initialize(body1, body2, anchorA, anchorB)
        dfn.length = (self.b2.b2Vec2(*anchorA) - self.b2.b2Vec2(*anchorB)).length
        dfn.frequencyHz = 4.0
        dfn.dampingRatio = 0.5
        return dfn

    def distance_asserts(self, dfn, joint):
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")
        self.check(dfn, joint, "length")
        self.check(dfn, joint, "frequencyHz", "frequency")
        self.check(dfn, joint, "dampingRatio")

    def distance_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- Rope joint ----
    def rope_definition(self, body1, body2, anchorA, anchorB, maxLength):
        dfn=self.b2.b2RopeJointDef(bodyA=body1, bodyB=body2, anchorA=anchorA, \
                        anchorB=anchorB, maxLength=maxLength)
        return dfn

    def rope_asserts(self, dfn, joint):
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")
        self.check(dfn, joint, "maxLength")

    def rope_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)
        joint.limitState

    # ---- pulley joint ----
    def pulley_definition(self, body1, body2):
        if body2.mass == 0 or body1.mass == 0:
            body1 = self.dbody2
            body2 = self.dbody1

        dfn=self.b2.b2PulleyJointDef()
        a, b = 2, 4
        y, L = 16, 12

        anchor1      =(body1.position.x, y + b)
        anchor2      =(body1.position.x, y + b)
        groundAnchor1=(body1.position.x, y + b + L)
        groundAnchor2=(body1.position.x, y + b + L)
        dfn.Initialize(body1, body2, groundAnchor1, groundAnchor2, anchor1, anchor2, 2.0)
        return dfn

    def pulley_asserts(self, dfn, joint):
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")
        self.check(dfn, joint, "groundAnchorA")
        self.check(dfn, joint, "groundAnchorB")
        self.check(dfn, joint, "ratio")

    def pulley_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- mouse joint ----
    def mouse_definition(self, body1, body2):
        dfn=self.b2.b2MouseJointDef()
        if body2.mass == 0:
            body2 = self.dbody1
            if body2 == body1:
                body1 = self.sbody1
        dfn.bodyA = body1
        dfn.bodyB = body2
        dfn.target = (2, 1)
        dfn.maxForce = 10
        return dfn

    def mouse_asserts(self, dfn, joint):
        self.check(dfn, joint, "target")
        self.check(dfn, joint, "maxForce")
        self.check(dfn, joint, "frequencyHz", "frequency")
        self.check(dfn, joint, "dampingRatio")

    def mouse_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- wheel joint ----
    def wheel_definition(self, body1, body2, anchor, axis):
        dfn=self.b2.b2WheelJointDef()
        dfn.Initialize(body1, body2, anchor, axis)
        dfn.motorSpeed = 0
        dfn.maxMotorForce = 100.0
        dfn.enableMotor = True
        dfn.lowerTranslation = -4.0
        dfn.upperTranslation = 4.0
        dfn.enableLimit = True
        return dfn

    def wheel_asserts(self, dfn, joint):
        self.check(dfn, joint, "motorSpeed")
        self.check(dfn, joint, "maxMotorTorque")
        self.check(dfn, joint, "enableMotor", "motorEnabled")
        self.check(dfn, joint, "bodyA")
        self.check(dfn, joint, "bodyB")
        self.check(dfn, joint, "frequencyHz", "springFrequencyHz")
        self.check(dfn, joint, "dampingRatio", "springDampingRatio")

    def wheel_checks(self, dfn, joint):
        # check to make sure they are at least accessible
        i = joint.anchorA
        i = joint.anchorB
        i = joint.speed
        i = joint.translation
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- weld joint ----
    def weld_definition(self, body1, body2):
        dfn=self.b2.b2WeldJointDef()
        dfn.bodyA = body1
        dfn.bodyB = body2
        return dfn

    def weld_asserts(self, dfn, joint):
        pass

    def weld_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    # ---- friction joint ----
    def friction_definition(self, body1, body2):
        dfn=self.b2.b2FrictionJointDef()
        dfn.bodyA = body1
        dfn.bodyB = body2
        dfn.localAnchorA = dfn.localAnchorB = (0,0)
        dfn.collideConnected = True
        dfn.maxForce = 10.0
        dfn.maxTorque = 20.0
        return dfn

    def friction_asserts(self, dfn, joint):
        self.check(dfn, joint, "maxForce")
        self.check(dfn, joint, "maxTorque")

    def friction_checks(self, dfn, joint):
        joint.GetReactionForce(1.0)
        joint.GetReactionTorque(1.0)

    def do_joint(self, name, init_args):
        '''test a joint by name'''
        get_dfn = getattr(self, "%s_definition"%name)
        asserts = getattr(self, "%s_asserts"%name)
        checks  = getattr(self, "%s_checks"%name)
        create_name=getattr(self.world, "Create%sJoint" % (name.capitalize()))

        for bodyA, bodyB in itertools.permutations( ( self.sbody1, self.sbody2, self.dbody1, self.dbody2), 2 ):
            try:
                dfn = get_dfn(body1=bodyA, body2=bodyB, **init_args)
            except Exception as ex:
                self._fail("%s: Failed on bodies %s and %s, joint definition (%s)" % (name, bodyA.userData, bodyB.userData, ex))

            kw_args = {}
            try:
                kw_args = dfn.to_kwargs()
                joint = create_name(**kw_args)
                self.world.DestroyJoint(joint)
            except Exception as ex:
                self._fail("%s: Failed on bodies %s and %s, joint creation by kwargs (%s) kw=%s" % (name, bodyA.userData, bodyB.userData, ex, kw_args))

            try:
                joint = self.world.CreateJoint(dfn)
            except Exception as ex:
                self._fail("%s: Failed on bodies %s and %s, joint creation (%s)" % (name, bodyA.userData, bodyB.userData, ex))

            try:
                asserts(dfn, joint)
            except Exception as ex:
                self.world.DestroyJoint(joint)
                raise

            try:
                checks(dfn, joint)
            except Exception as ex:
                self.world.DestroyJoint(joint)
                self._fail("%s: Failed on bodies %s and %s, joint checks (%s)" % (name, bodyA.userData, bodyB.userData, ex))

            try:
                self.step_world(5)
            except Exception as ex:
                # self.world.DestroyJoint(joint) # -> locked
                try:
                    # Ok, this will cause an assertion to go off during unwinding (in the b2StackAllocator deconstructor),
                    # so do it once, catch that, and then fail finally
                    self.fail()
                except AssertionError as ex:
                    self._fail("%s: Failed on bodies %s and %s, joint simulation (%s)" % (name, bodyA.userData, bodyB.userData, ex))

            try:
                self.world.DestroyJoint(joint)
            except Exception as ex:
                s=sys.exc_info()[1]
                self._fail("%s: Failed on bodies %s and %s joint deletion (%s)" % (name, bodyA.userData, bodyB.userData, s))

    # --- the actual tests ---
    def test_revolute(self):
        self.do_joint("revolute", { 'anchor' : (0, 12) })

    def test_prismatic(self):
        self.do_joint("prismatic", { 'anchor' : (0, 0), 'axis' : (1,0) })

    def test_distance(self):
        self.do_joint("distance", { 'anchorA' : (-10, 0), 'anchorB' : (-0.5,-0.5) })

    def test_rope(self):
        self.do_joint("rope", { 'anchorA' : (-10, 0), 'anchorB' : (-0.5,-0.5), 'maxLength' : 5.0 })

    def test_pulley(self):
        self.do_joint("pulley", {} )

    def test_mouse(self):
        self.do_joint("mouse", {} )

    def test_wheel(self):
        self.do_joint("wheel", { 'anchor' : (0, 8.5), 'axis' : (2,1) })

    def test_weld(self):
        self.do_joint("weld", {} )

    def test_friction(self):
        self.do_joint("friction", {} )

    def test_emptyjoint(self):
        try:
            self.world.CreateJoint( self.b2.b2RevoluteJointDef() )
        except ValueError:
            pass # good
        else:
            raise Exception('Empty joint should have raised an exception')

    def test_gear(self):
        # creates 2 revolute joints and then joins them, so it's done separately
        ground=self.world.CreateBody( self.b2.b2BodyDef() )
        shape=self.b2.b2EdgeShape(vertices=((50.0, 0.0), (-50.0, 0.0)))
        ground.CreateFixturesFromShapes(shapes=shape)

        body1=self.create_circle_body((-3, 12))
        body2=self.create_circle_body(( 0, 12))

        jd1=self.b2.b2RevoluteJointDef()
        jd1.Initialize(ground, body1, body1.position)
        joint1 = self.world.CreateJoint(jd1)

        jd2=self.b2.b2RevoluteJointDef()
        jd2.Initialize(ground, body2, body2.position)
        joint2 = self.world.CreateJoint(jd2)

        gjd=self.b2.b2GearJointDef()
        gjd.bodyA = body1
        gjd.bodyB = body2
        gjd.joint1 = joint1
        gjd.joint2 = joint2
        gjd.ratio  = 2.0

        gj = self.world.CreateJoint(gjd)

        self.step_world(10)

        self.check(gjd, gj, "ratio")
        gj.GetReactionForce(1.0)
        gj.GetReactionTorque(1.0)

        self.world.DestroyJoint(gj)
        self.world.DestroyJoint(joint2)
        self.world.DestroyJoint(joint1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

"""
Global Keys:
    F1     - toggle menu (can greatly improve fps)
    Space  - shoot projectile
    Z/X    - zoom
    Escape - quit

Other keys can be set by the individual test.

Mouse:
    Left click  - select/drag body (creates mouse joint)
    Right click - pan
    Shift+Left  - drag to create a directed projectile
    Scroll      - zoom

"""

from __future__ import (print_function, absolute_import, division)
import sys
import warnings

import pygame
from pygame.locals import (QUIT, KEYDOWN, KEYUP, MOUSEBUTTONDOWN,
                           MOUSEBUTTONUP, MOUSEMOTION, KMOD_LSHIFT)

from ..framework import (FrameworkBase, Keys)
from ..settings import fwSettings
from Box2D import (b2DrawExtended, b2Vec2)

try:
    from .pygame_gui import (fwGUI, gui)
    GUIEnabled = True
except Exception as ex:
    print('Unable to load PGU; menu disabled.')
    print('(%s) %s' % (ex.__class__.__name__, ex))
    GUIEnabled = False


class PygameDraw(b2DrawExtended):
    """
    This debug draw class accepts callbacks from Box2D (which specifies what to
    draw) and handles all of the rendering.

    If you are writing your own game, you likely will not want to use debug
    drawing.  Debug drawing, as its name implies, is for debugging.
    """
    surface = None
    axisScale = 10.0

    def __init__(self, test=None, **kwargs):
        b2DrawExtended.__init__(self, **kwargs)
        self.flipX = False
        self.flipY = True
        self.convertVertices = True
        self.test = test

    def StartDraw(self):
        self.zoom = self.test.viewZoom
        self.center = self.test.viewCenter
        self.offset = self.test.viewOffset
        self.screenSize = self.test.screenSize

    def EndDraw(self):
        pass

    def DrawPoint(self, p, size, color):
        """
        Draw a single point at point p given a pixel size and color.
        """
        self.DrawCircle(p, size / self.zoom, color, drawwidth=0)

    def DrawAABB(self, aabb, color):
        """
        Draw a wireframe around the AABB with the given color.
        """
        points = [(aabb.lowerBound.x, aabb.lowerBound.y),
                  (aabb.upperBound.x, aabb.lowerBound.y),
                  (aabb.upperBound.x, aabb.upperBound.y),
                  (aabb.lowerBound.x, aabb.upperBound.y)]

        pygame.draw.aalines(self.surface, color, True, points)

    def DrawSegment(self, p1, p2, color):
        """
        Draw the line segment from p1-p2 with the specified color.
        """
        pygame.draw.aaline(self.surface, color.bytes, p1, p2)

    def DrawTransform(self, xf):
        """
        Draw the transform xf on the screen
        """
        p1 = xf.position
        p2 = self.to_screen(p1 + self.axisScale * xf.R.x_axis)
        p3 = self.to_screen(p1 + self.axisScale * xf.R.y_axis)
        p1 = self.to_screen(p1)
        pygame.draw.aaline(self.surface, (255, 0, 0), p1, p2)
        pygame.draw.aaline(self.surface, (0, 255, 0), p1, p3)

    def DrawCircle(self, center, radius, color, drawwidth=1):
        """
        Draw a wireframe circle given the center, radius, axis of orientation
        and color.
        """
        radius *= self.zoom
        if radius < 1:
            radius = 1
        else:
            radius = int(radius)

        pygame.draw.circle(self.surface, color.bytes,
                           center, radius, drawwidth)

    def DrawSolidCircle(self, center, radius, axis, color):
        """
        Draw a solid circle given the center, radius, axis of orientation and
        color.
        """
        radius *= self.zoom
        if radius < 1:
            radius = 1
        else:
            radius = int(radius)

        pygame.draw.circle(self.surface, (color / 2).bytes + [127],
                           center, radius, 0)
        pygame.draw.circle(self.surface, color.bytes, center, radius, 1)
        pygame.draw.aaline(self.surface, (255, 0, 0), center,
                           (center[0] - radius * axis[0],
                            center[1] + radius * axis[1]))

    def DrawPolygon(self, vertices, color):
        """
        Draw a wireframe polygon given the screen vertices with the specified color.
        """
        if not vertices:
            return

        if len(vertices) == 2:
            pygame.draw.aaline(self.surface, color.bytes,
                               vertices[0], vertices)
        else:
            pygame.draw.polygon(self.surface, color.bytes, vertices, 1)

    def DrawSolidPolygon(self, vertices, color):
        """
        Draw a filled polygon given the screen vertices with the specified color.
        """
        if not vertices:
            return

        if len(vertices) == 2:
            pygame.draw.aaline(self.surface, color.bytes,
                               vertices[0], vertices[1])
        else:
            pygame.draw.polygon(
                self.surface, (color / 2).bytes + [127], vertices, 0)
            pygame.draw.polygon(self.surface, color.bytes, vertices, 1)

    # the to_screen conversions are done in C with b2DrawExtended, leading to
    # an increase in fps.
    # You can also use the base b2Draw and implement these yourself, as the
    # b2DrawExtended is implemented:
    # def to_screen(self, point):
    #     """
    #     Convert from world to screen coordinates.
    #     In the class instance, we store a zoom factor, an offset indicating where
    #     the view extents start at, and the screen size (in pixels).
    #     """
    #     x=(point.x * self.zoom)-self.offset.x
    #     if self.flipX:
    #         x = self.screenSize.x - x
    #     y=(point.y * self.zoom)-self.offset.y
    #     if self.flipY:
    #         y = self.screenSize.y-y
    #     return (x, y)


class PygameFramework(FrameworkBase):
    TEXTLINE_START = 30

    def setup_keys(self):
        keys = [s for s in dir(pygame.locals) if s.startswith('K_')]
        for key in keys:
            value = getattr(pygame.locals, key)
            setattr(Keys, key, value)

    def __reset(self):
        # Screen/rendering-related
        self._viewZoom = 10.0
        self._viewCenter = None
        self._viewOffset = None
        self.screenSize = None
        self.rMouseDown = False
        self.textLine = 30
        self.font = None
        self.fps = 0

        # GUI-related (PGU)
        self.gui_app = None
        self.gui_table = None
        self.setup_keys()

    def __init__(self):
        super(PygameFramework, self).__init__()

        self.__reset()
        if fwSettings.onlyInit:  # testing mode doesn't initialize pygame
            return

        print('Initializing pygame framework...')
        # Pygame Initialization
        pygame.init()
        caption = "Python Box2D Testbed - " + self.name
        pygame.display.set_caption(caption)

        # Screen and debug draw
        self.screen = pygame.display.set_mode((640, 480))
        self.screenSize = b2Vec2(*self.screen.get_size())

        self.renderer = PygameDraw(surface=self.screen, test=self)
        self.world.renderer = self.renderer

        try:
            self.font = pygame.font.Font(None, 15)
        except IOError:
            try:
                self.font = pygame.font.Font("freesansbold.ttf", 15)
            except IOError:
                print("Unable to load default font or 'freesansbold.ttf'")
                print("Disabling text drawing.")
                self.Print = lambda *args: 0
                self.DrawStringAt = lambda *args: 0

        # GUI Initialization
        if GUIEnabled:
            self.gui_app = gui.App()
            self.gui_table = fwGUI(self.settings)
            container = gui.Container(align=1, valign=-1)
            container.add(self.gui_table, 0, 0)
            self.gui_app.init(container)

        self.viewCenter = (0, 20.0)
        self.groundbody = self.world.CreateBody()

    def setCenter(self, value):
        """
        Updates the view offset based on the center of the screen.

        Tells the debug draw to update its values also.
        """
        self._viewCenter = b2Vec2(*value)
        self._viewCenter *= self._viewZoom
        self._viewOffset = self._viewCenter - self.screenSize / 2

    def setZoom(self, zoom):
        self._viewZoom = zoom

    viewZoom = property(lambda self: self._viewZoom, setZoom,
                        doc='Zoom factor for the display')
    viewCenter = property(lambda self: self._viewCenter / self._viewZoom, setCenter,
                          doc='Screen center in camera coordinates')
    viewOffset = property(lambda self: self._viewOffset,
                          doc='The offset of the top-left corner of the screen')

    def checkEvents(self):
        """
        Check for pygame events (mainly keyboard/mouse events).
        Passes the events onto the GUI also.
        """
        for event in pygame.event.get():
            if event.type == QUIT or (event.type == KEYDOWN and event.key == Keys.K_ESCAPE):
                return False
            elif event.type == KEYDOWN:
                self._Keyboard_Event(event.key, down=True)
            elif event.type == KEYUP:
                self._Keyboard_Event(event.key, down=False)
            elif event.type == MOUSEBUTTONDOWN:
                p = self.ConvertScreenToWorld(*event.pos)
                if event.button == 1:  # left
                    mods = pygame.key.get_mods()
                    if mods & KMOD_LSHIFT:
                        self.ShiftMouseDown(p)
                    else:
                        self.MouseDown(p)
                elif event.button == 2:  # middle
                    pass
                elif event.button == 3:  # right
                    self.rMouseDown = True
                elif event.button == 4:
                    self.viewZoom *= 1.1
                elif event.button == 5:
                    self.viewZoom /= 1.1
            elif event.type == MOUSEBUTTONUP:
                p = self.ConvertScreenToWorld(*event.pos)
                if event.button == 3:  # right
                    self.rMouseDown = False
                else:
                    self.MouseUp(p)
            elif event.type == MOUSEMOTION:
                p = self.ConvertScreenToWorld(*event.pos)

                self.MouseMove(p)

                if self.rMouseDown:
                    self.viewCenter -= (event.rel[0] /
                                        5.0, -event.rel[1] / 5.0)

            if GUIEnabled:
                self.gui_app.event(event)  # Pass the event to the GUI

        return True

    def run(self):
        """
        Main loop.

        Continues to run while checkEvents indicates the user has
        requested to quit.

        Updates the screen and tells the GUI to paint itself.
        """

        # If any of the test constructors update the settings, reflect
        # those changes on the GUI before running
        if GUIEnabled:
            self.gui_table.updateGUI(self.settings)

        running = True
        clock = pygame.time.Clock()
        while running:
            running = self.checkEvents()
            self.screen.fill((0, 0, 0))

            # Check keys that should be checked every loop (not only on initial
            # keydown)
            self.CheckKeys()

            # Run the simulation loop
            self.SimulationLoop()

            if GUIEnabled and self.settings.drawMenu:
                self.gui_app.paint(self.screen)

            pygame.display.flip()
            clock.tick(self.settings.hz)
            self.fps = clock.get_fps()

        self.world.contactListener = None
        self.world.destructionListener = None
        self.world.renderer = None

    def _Keyboard_Event(self, key, down=True):
        """
        Internal keyboard event, don't override this.

        Checks for the initial keydown of the basic testbed keys. Passes the unused
        ones onto the test via the Keyboard() function.
        """
        if down:
            if key == Keys.K_z:       # Zoom in
                self.viewZoom = min(1.1 * self.viewZoom, 50.0)
            elif key == Keys.K_x:     # Zoom out
                self.viewZoom = max(0.9 * self.viewZoom, 0.02)
            elif key == Keys.K_SPACE:  # Launch a bomb
                self.LaunchRandomBomb()
            elif key == Keys.K_F1:    # Toggle drawing the menu
                self.settings.drawMenu = not self.settings.drawMenu
            elif key == Keys.K_F2:    # Do a single step
                self.settings.singleStep = True
                if GUIEnabled:
                    self.gui_table.updateGUI(self.settings)
            else:              # Inform the test of the key press
                self.Keyboard(key)
        else:
            self.KeyboardUp(key)

    def CheckKeys(self):
        """
        Check the keys that are evaluated on every main loop iteration.
        I.e., they aren't just evaluated when first pressed down
        """

        pygame.event.pump()
        self.keys = keys = pygame.key.get_pressed()
        if keys[Keys.K_LEFT]:
            self.viewCenter -= (0.5, 0)
        elif keys[Keys.K_RIGHT]:
            self.viewCenter += (0.5, 0)

        if keys[Keys.K_UP]:
            self.viewCenter += (0, 0.5)
        elif keys[Keys.K_DOWN]:
            self.viewCenter -= (0, 0.5)

        if keys[Keys.K_HOME]:
            self.viewZoom = 1.0
            self.viewCenter = (0.0, 20.0)

    def Step(self, settings):
        if GUIEnabled:
            # Update the settings based on the GUI
            self.gui_table.updateSettings(self.settings)

        super(PygameFramework, self).Step(settings)

        if GUIEnabled:
            # In case during the step the settings changed, update the GUI reflecting
            # those settings.
            self.gui_table.updateGUI(self.settings)

    def ConvertScreenToWorld(self, x, y):
        return b2Vec2((x + self.viewOffset.x) / self.viewZoom,
                      ((self.screenSize.y - y + self.viewOffset.y) / self.viewZoom))

    def DrawStringAt(self, x, y, str, color=(229, 153, 153, 255)):
        """
        Draw some text, str, at screen coordinates (x, y).
        """
        self.screen.blit(self.font.render(str, True, color), (x, y))

    def Print(self, str, color=(229, 153, 153, 255)):
        """
        Draw some text at the top status lines
        and advance to the next line.
        """
        self.screen.blit(self.font.render(
            str, True, color), (5, self.textLine))
        self.textLine += 15

    def Keyboard(self, key):
        """
        Callback indicating 'key' has been pressed down.
        The keys are mapped after pygame's style.

         from framework import Keys
         if key == Keys.K_z:
             ...
        """
        pass

    def KeyboardUp(self, key):
        """
        Callback indicating 'key' has been released.
        See Keyboard() for key information
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, main)
from Box2D import (b2EdgeShape, b2FixtureDef, b2PolygonShape)


class ConveyorBelt (Framework):
    name = "ConveyorBelt"

    def __init__(self):
        Framework.__init__(self)

        self.using_contacts = True
        ground = self.world.CreateStaticBody(
            shapes=[b2EdgeShape(vertices=[(-20, 0), (20, 0)])],
        )

        # Platform
        self.platform = self.world.CreateStaticBody(
            position=(-5, 5),
            allowSleep=False,
            fixtures=b2FixtureDef(friction=0.8,
                                  shape=b2PolygonShape(box=(10.0, 5.0)),),
        )

        self.platform_fixture = self.platform.fixtures[0]

        # Boxes
        for i in range(5):
            self.platform = self.world.CreateDynamicBody(
                position=(-10.0 + 2.0 * i, 7.0),
                fixtures=b2FixtureDef(density=20.0,
                                      shape=b2PolygonShape(box=(0.5, 0.5)),),
            )

    def PreSolve(self, contact, old_manifold):
        Framework.PreSolve(self, contact, old_manifold)

        fixture_a, fixture_b = contact.fixtureA, contact.fixtureB

        if fixture_a == self.platform_fixture:
            contact.tangentSpeed = 5.0
        elif fixture_b == self.platform_fixture:
            contact.tangentSpeed = -5.0

if __name__ == "__main__":
    main(ConveyorBelt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version Copyright (c) 2010 kne / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2CircleShape, b2FixtureDef, b2LoopShape, b2PolygonShape,
                   b2RevoluteJointDef, b2_pi)


class Pinball (Framework):
    name = "Pinball"
    description = ('This tests bullet collision and provides an example of a gameplay scenario.\n'
                   'Press A to control the flippers.')
    bodies = []
    joints = []

    def __init__(self):
        super(Pinball, self).__init__()

        # The ground
        ground = self.world.CreateBody(
            shapes=b2LoopShape(vertices=[(0, -2), (8, 6),
                                         (8, 20), (-8, 20),
                                         (-8, 6)]),
        )

        # Flippers
        p1, p2 = (-2, 0), (2, 0)

        fixture = b2FixtureDef(shape=b2PolygonShape(box=(1.75, 0.1)), density=1)
        flipper = {'fixtures': fixture}

        self.leftFlipper = self.world.CreateDynamicBody(
            position=p1,
            **flipper
        )
        self.rightFlipper = self.world.CreateDynamicBody(
            position=p2,
            **flipper
        )

        rjd = b2RevoluteJointDef(
            bodyA=ground,
            bodyB=self.leftFlipper,
            localAnchorA=p1,
            localAnchorB=(0, 0),
            enableMotor=True,
            enableLimit=True,
            maxMotorTorque=1000,
            motorSpeed=0,
            lowerAngle=-30.0 * b2_pi / 180.0,
            upperAngle=5.0 * b2_pi / 180.0,
        )

        self.leftJoint = self.world.CreateJoint(rjd)

        rjd.motorSpeed = 0
        rjd.localAnchorA = p2
        rjd.bodyB = self.rightFlipper
        rjd.lowerAngle = -5.0 * b2_pi / 180.0
        rjd.upperAngle = 30.0 * b2_pi / 180.0
        self.rightJoint = self.world.CreateJoint(rjd)

        # Ball
        self.ball = self.world.CreateDynamicBody(
            fixtures=b2FixtureDef(
                shape=b2CircleShape(radius=0.2),
                density=1.0),
            bullet=True,
            position=(1, 15))

        self.pressed = False

    def Keyboard(self, key):
        if key == Keys.K_a:
            self.pressed = True

    def KeyboardUp(self, key):
        if key == Keys.K_a:
            self.pressed = False

    def Step(self, settings):
        if self.pressed:
            self.leftJoint.motorSpeed = 20
            self.rightJoint.motorSpeed = -20
        else:
            self.leftJoint.motorSpeed = -10
            self.rightJoint.motorSpeed = 10
        super(Pinball, self).Step(settings)

if __name__ == "__main__":
    main(Pinball)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from Box2D import b2Color

class testColor (unittest.TestCase):
    def checkAlmostEqual(self, v1, v2, msg, places=3):
        for i, (a, b) in enumerate(zip(v1, v2)):
            self.assertAlmostEqual(a, b, places=places, 
                    msg="(index %d) %s, a=%f b=%f from %s, %s" % (i, msg, a, b, v1, v2))

    def test_color(self):
        x, y, z = 1.0, 2.0, 3.0
        c1 = b2Color(x, y, z)
        c2 = b2Color(z, y, x)

        c = c1 + c2
        self.checkAlmostEqual(c, (x+z, y+y, z+x), msg='b2Color +')
        c = c1 - c2
        self.checkAlmostEqual(c, (x-z, y-y, z-x), msg='b2Color -')
        c = 2.0 * c1
        self.checkAlmostEqual(c, (x+x, y+y, z+z), msg='float * b2Color')
        c = c1 * 2.0
        self.checkAlmostEqual(c, (x+x, y+y, z+z), msg='b2Color * float')
        c = c1 / 2.0
        self.checkAlmostEqual(c, (x/2.0, y/2.0, z/2.0), msg='b2Color / float')

        c = c1.copy()
        c *= 2.0
        self.checkAlmostEqual(c, (x+x, y+y, z+z), msg='b2Color *= float')
        c = b2Color(c1)
        c /= 2.0
        self.checkAlmostEqual(c, (x/2.0, y/2.0, z/2.0), msg='b2Color /= float')
        c1 += (1.0, 1.0, 1.0)
        self.checkAlmostEqual(c1, (x+1, y+1, z+1), msg='b2Color +=')
        c1 -= (1.0, 1.0, 1.0)
        self.checkAlmostEqual(c1, (x, y, z), msg='b2Color -=')

        bytes=b2Color(1, 1, 1).bytes
        self.assertEqual(bytes, [255,255,255], msg='bytes (1,1,1)=>%s'%bytes)
        bytes=b2Color(0, 0, 0).bytes
        self.assertEqual(bytes, [0,0,0], msg='bytes (1,1,1)=>%s'%bytes)

        self.assertEqual(c1[0], x)
        self.assertEqual(c1[1], y)
        self.assertEqual(c1[2], z)

        c1.list = (x*2, y*2, z*2)
        self.checkAlmostEqual(c1, (x+x, y+y, z+z), msg='b2Color.list')

if __name__ == '__main__':
    unittest.main()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# C++ version Copyright (c) 2006-2007 Erin Catto http://www.box2d.org
# Python version by Ken Lauer / sirkne at gmail dot com
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
# 1. The origin of this software must not be misrepresented; you must not
# claim that you wrote the original software. If you use this software
# in a product, an acknowledgment in the product documentation would be
# appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
# misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

from Box2D.examples.framework import (Framework, Keys, main)
from Box2D import (b2CircleShape, b2EdgeShape, b2FixtureDef, b2PolygonShape)


class VerticalStack (Framework):
    name = "Vertical Stack"
    description = ("Tests the stability of stacking circles and boxes\n"
                   "Press B to launch a horizontal bullet")
    bullet = None

    def __init__(self):
        super(VerticalStack, self).__init__()

        columns = 5
        rows = 16

        ground = self.world.CreateStaticBody(
            shapes=[
                b2EdgeShape(vertices=[(-40, 0), (40, 0)]),
                b2EdgeShape(vertices=[(20, 0), (20, 20)]),
            ]
        )

        box = b2FixtureDef(
            shape=b2PolygonShape(box=(0.5, 0.5)),
            density=1,
            friction=0.3)
        circle = b2FixtureDef(
            shape=b2CircleShape(radius=0.5),
            density=1,
            friction=0.3)

        box_start = -10
        box_space = 2.5
        circle_start = 8
        circle_space = 2.5
        for j in range(columns):
            for i in range(rows):
                self.world.CreateDynamicBody(
                    fixtures=box,
                    position=(box_start + box_space * j, 0.752 + 1.54 * i)
                )
                self.world.CreateDynamicBody(
                    fixtures=circle,
                    position=(circle_start + circle_space *
                              j, 0.752 + 1.54 * i)
                )

    def Step(self, settings):
        super(VerticalStack, self).Step(settings)

    def Keyboard(self, key):
        if key == Keys.K_b:
            if self.bullet:
                self.world.DestroyBody(self.bullet)
                self.bullet = None
            circle = b2FixtureDef(
                shape=b2CircleShape(radius=0.25),
                density=20,
                restitution=0.05)
            self.bullet = self.world.CreateDynamicBody(
                position=(-31, 5),
                bullet=True,
                fixtures=circle,
                linearVelocity=(400, 0),
            )

if __name__ == "__main__":
    main(VerticalStack)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
import Box2D as b2
import sys

class testEdgeChain (unittest.TestCase):
    def setUp(self):
        pass

    def test_create_edge_chain(self):
        world = b2.b2World()

        ground = world.CreateBody(position=(0, 20))

        try:
            ground.CreateEdgeChain([])
        except ValueError:
            pass #good
        except Exception:
            self.fail("Failed to create empty edge chain (%s)" % sys.exc_info()[1])

        try:
            ground.CreateEdgeChain(
                                [ (-20,-20),
                                  (-20, 20),
                                  ( 20, 20),
                                  ( 20,-20),
                                  (-20,-20) ]
                                )
        except Exception:
            self.fail("Failed to create valid edge chain (%s)"  % sys.exc_info()[1])

    def test_b2EdgeShape(self):
        world = b2.b2World()

        v1=(-10.0, 0.0)
        v2=(-7.0, -1.0)
        v3=(-4.0, 0.0)

        ground=world.CreateStaticBody(shapes=
                [b2.b2EdgeShape(vertices=[None, v1, v2, v3])])

if __name__ == '__main__':
    unittest.main()

